#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2010-2011 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           dialogs.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import utils
import pygame
from pygame.constants import *
from base import Widget
from buttons import *
from SPConstants import *
from text import TextView
from SPSpriteUtils import SPSprite, SPGroup
from funcs import make_dialog_bg_dynamic
import subprocess

class Dialog(Widget):
    """Modal dialog which displays text and has one or two buttons.
    The dialog has it's own eventloop.
    Usage:
    dlg = Dialog(txt, (10, 200),buttons=["Cancel", "OK"], title='Information')
    dlg.run() # this blocks any other events loops
    answer = dlg.get_result()
    dlg.erase_sprite()
    """
    def __init__(self, txt, pos=None, fsize=12, butfsize=20, buttons=['OK'], title='', \
                 dialogwidth=0, dialogheight=0, name='', fbold=True):
        """screen - current pygame screen surface
        txt - can be a string or a list of strings or a SPSprite object.
        pos - is a tuple with x,y grid numbers. If not set the dialog is centered.
        fsize -  the fontsize (integer)
        buttons - a list with strings for the buttons (max two)
        dialog width and hight - 0 means dynamic.
        name - to identify this object (optional) 
        """
        Widget.__init__(self)
        self.scrclip = self.screen.get_clip()
        self.screen.set_clip((0, 0, 800, 600))   
        self.dim = utils.Dimmer()
        self.dim.dim(darken_factor=140, color_filter=CORNFLOWERBLUE)
        self.orgscreen = self.screen.convert()
        self.orgbackgr = self.backgr.convert()
        self.txt = txt 
        self.pos = pos
        self.fsize = fsize
        self.buttons = buttons
        self.title = title
        self.dialogwidth = dialogwidth
        self.dialogheight = dialogheight
        self.name = name
        self.fbold = fbold
        self.result = None
        self.runloop = True
        self.buttons_list = []
        self.butspace = 0
        self.padding = 8
        self.actives = SPGroup(self.screen, self.backgr)
        self.actives.set_onematch(True)
        for buttxt in self.buttons:
            b = ButtonDynamic(buttxt, (380,250), butfsize, padding=8, name=buttxt,\
                               fgcol=BLACK, colorname='green')
            b.set_use_current_background(True)
            b.connect_callback(self.cbf, MOUSEBUTTONUP, buttxt)
            self.actives.add(b)
            self.buttons_list.append(b)# keep the users order for showing (see below)
            self.butspace += b.rect.w + 8
        if not self.dialogwidth:
            self.dialogwidth = self.butspace
        self.ImShowed = False
        self._setup()
    
    def _setup(self):
        # get all the surfaces
        title = utils.char2surf(self.title, 24, ttf=TTF, fcol=self.THEME['dialog_title_fg_color'], bold=True)
        
        if type(self.txt) == types.ListType or type(self.txt) in types.StringTypes:
            tv = TextView(self.txt, (0, 0), pygame.Rect(0, 0, self.dialogwidth, 400),\
                           bgcol='trans',fsize=self.fsize ,\
                          fgcol=self.THEME['dialog_fg_color'], autofit=True, bold=self.fbold)
        else: # we assume a SPWidget.Widget object
            tv = self.txt
        b = self.buttons_list[0]
        # start building dialog
        if self.dialogwidth and self.dialogheight:
            r = pygame.Rect(0, 0, self.dialogwidth, self.dialogheight)
        else:
            r = pygame.Rect(0, 0, max(self.butspace,(tv.rect.w + (self.padding * 2)),100), \
                        title.get_rect().h + 8 + b.rect.h + tv.rect.h + (self.padding*3))
        if r.w > 750:
            r.w = 750
        if r.h > 530:
            r.h = 530
        dlgsurf, dlgsizes = make_dialog_bg_dynamic(r.w+50, r.h+70, self.THEME)
        
        r = dlgsurf.get_rect()
        if not self.pos:
            y = max(10, (600 - r.h) / 2)
            x = (800 - r.w) / 2
            self.pos = (x, y)

        y = 10
        x = 20
        dlgsurf.blit(title, (x,y))
        dlgsurf.blit(tv.image, (x, dlgsizes['title_area'].h))
        
        Widget.__init__(self, dlgsurf)
        
        self.image = dlgsurf

        self.rect.move_ip(self.pos)
        for k, v in dlgsizes.items():
            dlgsizes[k].move_ip(self.pos)
        x = self.rect.left + ((self.rect.w - (self.butspace + 16 * len(self.buttons_list))) / 2)
        y = self.rect.bottom - self.buttons_list[0].rect.h *2
        if len(self.buttons_list) == 2:
            x = self.rect.left + 16
            self.buttons_list[0].moveto((x, y), True)
            x = self.rect.right - (self.buttons_list[1].rect.w + 16)
            self.buttons_list[1].moveto((x, y), True)
        else:
            for b in self.buttons_list:
                b.moveto((x, y), True)
                x += b.rect.w + 16
        self.dlgsizes = dlgsizes
            
    def cbf(self, widget, event, data):
        self.result = data 
        self.runloop = False
        
    def get_result(self):
        return self.result
    
    def get_action_area(self):
        return self.dlgsizes
    
    def show(self, guests=None):
        if guests:
            self.actives.add(guests)
        self.display_sprite()
        for b in self.actives:
            b.display_sprite()
        self.ImShowed = True
    
    def run(self, guests=None):
        if not self.ImShowed:
            if guests:
                self.actives.add(guests)
            self.display_sprite()
            for b in self.actives:
                b.display_sprite()
        clock = pygame.time.Clock()
        while self.runloop:
            clock.tick(30)
            pygame.event.pump()
            events = pygame.event.get()
            for event in events:
                if event.type is KEYDOWN and event.key is K_ESCAPE:
                    self.runloop = False
                else:
                    self.actives.update(event)
        pygame.event.clear()
        self.erase_sprite()
        self.screen.blit(self.orgscreen, (0, 0))
        self.backgr.blit(self.orgbackgr, (0, 0))
        self.dim.undim()
        self.screen.set_clip(self.scrclip)
        if hasattr(self, 'reset_children'):
            # I'm a DialogWindow
            self.reset_children()

class DialogWindow(Dialog):
    """Modal dialog which displays sprites and has one or two buttons of it's own.
    The dialog has it's own eventloop.
    This differs from a standard dialog which can only display a surface. 
    
    This dialog can display and handle sprite objects which will be placed relativly
    to the dialogs' position.
    So if the dialog is displayed at 100,100 all sprites will be moved by 100,100.
    You must layout the sprites properly and make sure the dialog is big enough.
    The dialog will move all sprites by the dialogs display offset.
    
    The dialog calls the update method on all sprites in the eventloop.
    You can connect your sprites to callback functions to collect data or whatever.
    When the dialog quits all the sprite objects still exists so you can query them
    for data or states.
     
    Usage: TODO
    """
    def __init__(self, sprites, pos=None, butfsize=20,dialogheight=200, dialogwidth=200,\
              buttons=['OK'], title='', name=''):
        """screen - current pygame screen surface
        sprites - a list with all the sprites to display on the 'back' surface.
                The sprites must have the name attribute and position relative to the dialog set.
                You should connect the sprites to callbacks to get results.
        pos - is a tuple with x,y grid numbers. If not set the dialog is centered.
        buttons - a list with strings for the buttons (max two)
        name - to identify this object (optional) 
        """
        Dialog.__init__(self, ' ', pos=pos, fsize=12, butfsize=butfsize, buttons=buttons,\
                         title=title, dialogwidth=dialogwidth, dialogheight=dialogheight,\
                         name=name, fbold=True)
        self.show()
        self._org_pos = []
        x, y = self.get_action_area()['action_area'].topleft
        for s in sprites:
            s.set_use_current_background(True)
            sx, sy = s.get_sprite_pos()
            self._org_pos.append((s,(sx,sy)))
            s.moveto((sx + x,sy + y))
            s.display_sprite()
        self.sprites = sprites 
        self.actives.add(sprites)
     
    def reset_children(self):
        """This wil reset the positions from the sprites back to their original postions."""
        for s, pos in self._org_pos:
            s.moveto(pos) 

        
class MenuBar(Widget):
    def __init__(self, rect, actives, cbf, dice_cbf,lock, usestar=False, \
                 usegraph=False, usedice=False, volume_level=50):
        Widget.__init__(self)
        self.buttons_ypos = self.rect.top+8
        bname = 'core_info_button.png'
        hbname = 'core_info_button_ro.png'
        p = os.path.join(THEMESPATH, self.THEME['theme'],bname )
        hp = os.path.join(THEMESPATH, self.THEME['theme'],hbname )
        pos = (CORE_BUTTONS_XCOORDS[0], self.buttons_ypos)
        if not os.path.exists(p):
            p = os.path.join(DEFAULTTHEMESPATH, bname)
        if not os.path.exists(hp):
            hp = None
        if hp:
            self.infobutton = TransImgButton(p, hp,pos, name='Info')
        else:
            self.infobutton = ImgButton(p, pos, name='Info')
        self.infobutton.mouse_hover_leave_action = True
        self.infobutton.connect_callback(cbf, MOUSEBUTTONUP, 'Info')
        
        bname = 'core_quit_button.png'
        hbname = 'core_quit_button_ro.png'
        p = os.path.join(THEMESPATH, self.THEME['theme'],bname )
        hp = os.path.join(THEMESPATH, self.THEME['theme'],hbname )
        pos = (CORE_BUTTONS_XCOORDS[8], self.buttons_ypos)
        if not os.path.exists(p):
            p = os.path.join(DEFAULTTHEMESPATH, bname)
        if not os.path.exists(hp):
            hp = None
        if hp:
            self.quitbutton = TransImgButton(p, hp,pos, name='Quit')
        else:
            self.quitbutton = ImgButton(p, pos, name='Quit')
        self.quitbutton.mouse_hover_leave_action = True
        self.quitbutton.connect_callback(cbf, MOUSEBUTTONUP, 'Quit')
        
        if volume_level == 0:
            bname = 'core_volmute_button.png'
            hbname = 'core_volmute_button_ro.png'
        else:
            bname = 'core_volume_button.png'
            hbname = 'core_volume_button_ro.png'
        p = os.path.join(THEMESPATH, self.THEME['theme'],bname )
        hp = os.path.join(THEMESPATH, self.THEME['theme'],hbname )
        pos = (CORE_BUTTONS_XCOORDS[7], self.buttons_ypos)
        if not os.path.exists(p):
            p = os.path.join(DEFAULTTHEMESPATH, bname)
        if not os.path.exists(hp):
            hp = None
        if hp:
            self.volumebutton = TransImgButton(p, hp,pos, name='Volume')
        else:
            self.volumebutton = ImgButton(p, pos, name='Volume')
        self.volumebutton.mouse_hover_leave_action = True
        self.volumebutton.connect_callback(cbf, MOUSEBUTTONUP, 'Volume')
        
        if usegraph:
            self.chartbutton = ChartButton((CORE_BUTTONS_XCOORDS[5], self.buttons_ypos), cbf)
            self.chartbutton.mouse_hover_leave_action = True
        else:
            self.chartbutton = None
        if usestar:
            self.dicebuttons = StarButton((CORE_BUTTONS_XCOORDS[1], self.buttons_ypos), \
                                                        actives, dice_cbf, lock, usedice)
        else:
            self.dicebuttons = DiceButtons((CORE_BUTTONS_XCOORDS[3], self.buttons_ypos),\
                                                        actives, dice_cbf, usedice)
        self.scoredisplay = ScoreDisplay((CORE_BUTTONS_XCOORDS[6], self.buttons_ypos+4), cbf=cbf)
    
    def get_buttons_posy(self):
        return self.buttons_ypos        
    def get_scoredisplay(self):
        return self.scoredisplay
    def get_infobutton(self):
        return self.infobutton
    def get_quitbutton(self):
        return self.quitbutton
    def get_chartbutton(self):
        return self.chartbutton
    def get_dicebuttons(self):
        return self.dicebuttons
    def get_volumebutton(self):
        return self.volumebutton


class Graph(Widget):
    """provides a SDL surface with a graph plotted onto.
    The surface will be, x*y, 502x302 and will hold up to 20 values.
    This can only be used to display scores from schoolsplay.
    """
    def __init__(self, data, level, headertext='', norm=None):
        """@data must be a list with tuples (score,date).
        @norm is a tuple containing the mu and sigma value.
        """   
        # dates are stored like this: 07-09-09_10:44:27
        # the data list is a list with tuples like (mu,sigma,date)
        self.logger = logging.getLogger("childsplay.SPWidgets_lgpl.Graph")
        # main surf is 500x300 the graph is a surf of 470x270.
        # 20 for the text bottom and left and 10 for a top and right offset
        gs = pygame.Surface((470, 270))# surf for the graph
        gs.fill((230, 230, 230))
        self.s = pygame.Surface((500, 400))# main surf
        self.s.fill((200, 200, 200))
        if not data:
            self.logger.warning("No data received to show in graph")
            ts = utils.char2surf(_("No data available for this level"), TTFSIZE + 8, ttf=TTF, split=35)
            y = 100
            for s in ts:
                self.s.blit(s, (20, y))
                y += s.get_height()
            Widget.__init__(self, self.s)
            return
        # border around the surf
        pygame.draw.rect(self.s, (100, 100, 100), self.s.get_rect().inflate(-1, -1), 2)
        
        # draw mu line and sigma deviation as a transparent box.
        if norm:
            mu, sigma = norm
            # line
            pygame.draw.line(gs, DARK_BLUE, (0, 270-mu * 27), (470, 270-mu * 27), 3)
            # deviation box, transparent blue
            top_y = mu + (mu / 100.0 * 30)
            bottom_y = mu - (mu / 100.0 * 30)
            darken = pygame.Surface((470, int((top_y-bottom_y) * 27)))
            darken.fill(BLUE)
            darken_factor = 64
            darken.set_alpha(darken_factor)
            gs.blit(darken, (0, 270-top_y * 27))
            
        # draw y-axes graph lines, initial y offset is 30 pixels to leave room for text.
        i = 10
        for y in range(0, 270, 27):
            pygame.draw.line(gs, (0, 0, 0), (0, y), (470, y), 1)
            ts = utils.char2surf(str(i), TTFSIZE-2, ttf=TTF, bold=True)
            i -= 1
            self.s.blit(ts, (4, y + 35))
        ts0 = utils.char2surf(headertext, TTFSIZE-2, ttf=TTF, bold=True)
        ts1 = utils.char2surf(_("Percentile scores for level %s") % level, TTFSIZE, ttf=TTF)
        self.s.blit(ts0, (10, 2))
        self.s.blit(ts1, (20, 20))
        
        # Draw x-axes data
        # determine the size of the x graph steps. This depends on the number of data items.
        # as we fill the x axe. Y step is always the same as the y axe values are always 0-10.
        # The maximum is 470 items, one pixel per data item.
        x_step = 470 / len(data)
        dotslist = []
        # draw dots and x-axes lines
        for x in range(0, 471-x_step, x_step):
            pygame.draw.line(gs, (0, 0, 0), (x, 270), (x, 0), 1)
            try:
                item = data.pop(0)
                #print item
                y = int(float(item[1]) * 27)
                #print "y position",y
                date = item[0]
                #print "date",date
            except (IndexError, TypeError):
                self.logger.exception("Troubles in the datalist, this shouldn't happen\n I try to continue but expect weird results :-)\n Traceback follows:")
                continue
            else:
                pygame.draw.circle(gs, DARK_BLUE, (x + 4, 270-y + 4), 4, 0)
                dotslist.append((x, 270-y))
                # blit date, rotated 90 degrees clockwise
                ts = utils.char2surf(date, TTFSIZE-5, ttf=TTF)
                # rotate 90 degrees
                v_ts = pygame.transform.rotate(ts, 270)
                self.s.blit(v_ts, (x + 20, 320))
        # connect the dots if there are more then one
        if len(dotslist) > 1:
            pygame.draw.lines(gs, BLUE, 0, dotslist, 2)
        pygame.draw.rect(gs, (0, 0, 0), gs.get_rect().inflate(-1, -1), 2)
        # finally blit the two surfaces
        self.s.blit(gs, (20, 40))
        Widget.__init__(self, self.s)
    
    def _calculate_position(self, pos):
        new_pos = pos# finish this
        return new_pos
    
    def get_surface(self):
        return self.image


class ExeCounter(Widget):
    # TODO: Finish this into a SPWidgets widget
    """Object that provides a, transparent, Pygame Surface that displays a simple 
    exercise counter in the form, done/todo.
    It uses two label objects for the display and provides some methods to 
    interact with these labels.
    The object places the labels in the menubar.
    """
    def __init__(self, pos, total, fsize=10, text=''):
        """The width of the widget is determined by the width of the headertext
        with a minimal width of four spaces and a maximum of fourteen characters.
        total - the total number of exercises.
        fsize - Fontsize for the text
        text - Headertext defaults to 'Exercises' (localized)
        """
        self.logger = logging.getLogger("childsplay.SPocwWidgets.ExeCounter")
        Widget.__init__(self)
        self.total = total
        self.done = 0
        if not text:
            text = _("Exercises")
        if len(text) > 14:#prevent to large label when localized
            text = text[:14]
        fgcol=self.THEME['execounter_fg_color']
        bgcol = self.THEME['execounter_bg_color']
        s0 = utils.char2surf(text, fsize=fsize, fcol=fgcol)
        txt = "%s/%s" % (self.done, self.total)
        
        self.lbl = Label(txt, (pos[0]+2, pos[1]+s0.get_height()), fsize=12, padding=4, border=0, fgcol=fgcol, bgcol=bgcol)
        
        r = pygame.Rect(0, 0,s0.get_rect().w+4, s0.get_rect().h +\
                                               self.lbl.rect.h+4)
        box, spam = get_boxes(bgcol, BLACK, r)
        
        box.blit(s0, (2, 2))
        self.image = box       
        Widget.__init__(self, box)
        self.rect.move_ip(pos)
        self.display_sprite()
        self.lbl.display_sprite()
    
    def increase_counter(self):
        """Increase the 'done' part of the counter.
        When the 'done' part is equal to the 'total' part no increase
        is performed."""
        if self.done == self.total:
            return
        self.done += 1
        text = "%s/%s" % (self.done, self.total)
        self.lbl.settext(text)
        self.lbl.display_sprite()
    
    def reset_counter(self, total):
        """Reset the total counter to @total.
        The 'done' part is reset to zero.
        """
        self.total = total
        self.done = -1
        self.increase_counter()
        
    
class ScoreDisplay(Label):
    """Displays a sore value of max 4 digits.
    It provides methods to set, increase, clear and return the scores."""
    def __init__(self, pos, score=0, cbf=None):
        self.scorestr = '%04d' % score
        # TODO: set fgcol and bgcol kwargs ?
        Label.__init__(self, self.scorestr, pos, fsize=20, padding=9, border=1)
        if cbf:
            self.connect_callback(cbf, MOUSEBUTTONUP, 'scoredisplay')
            # We disguise our selfs into a graphbutton
            self.name = 'Chart'
        else:
            self.name = 'ScoreDisplay'
        self.UseMe = True
    
    def _showme(self):
        if self.UseMe:
            self.display_sprite()
    
    def set_score(self, score=0):
        if score < 0:
            return
        if score > 9999:
            score = 9999
        self.scorestr = '%04d' % score
        self.settext(self.scorestr)
        self._showme()
    
    def increase_score(self, score):
        if int(self.scorestr) + score > 9999:
            score = 0
        if int(self.scorestr) + score < 0:
            score = 0
        self.scorestr = '%04d' % (int(self.scorestr) + int(score))
        self.settext(self.scorestr)
        self._showme()
        
    def clear_score(self):
        self.scorestr = '0000'
        self.settext('0000')
        self._showme()
        
    def return_score(self):
        return int(self.scorestr)

class VolumeAdjust(Widget):
    """Widget that provides a volume change widget with on both ends a small button
    to increase or decrease and mute the volume level and in the middle a percentage display.
    """
    def __init__(self, pos, volume=None, voice_unmute=True, defval=50):
        """defval - value between 0, silent, and 100, maximum, volume level."""
        Widget.__init__(self)
        self.logger = logging.getLogger("childsplay.SPWidgets.VolumeAdjust")
        self.logger.debug("VolumeAdjust called with volume level %s" % volume)
        self.soundcheck = utils.load_sound(os.path.join(ACTIVITYDATADIR, 'CPData','volumecheck.wav'))
        self.theme = self.THEME['theme']
        if volume or int(volume) == 0:
            #print "we have volume"
            self.volume = int(volume)
        elif self.WEHAVEAUMIX:
            #print "we have aumix"
            self.volume = int(self.WEHAVEAUMIX)
        else:
            #print "else defval"
            self.volume = int(defval)
        self.logger.debug("setting volume string to %s" % self.volume)
        self.volstr = '%02d' % self.volume + "%"
        # TODO: set fgcol and bgcol kwargs ?
        self.logger.debug("setting voice unmute to %s" % voice_unmute)
        imgup = os.path.join(THEMESPATH, self.theme,'core_volup_button.png')
        imgup_ro = os.path.join(THEMESPATH, self.theme,'core_volup_button_ro.png')
        imgdown = os.path.join(THEMESPATH, self.theme,'core_voldown_button.png')
        imgdown_ro = os.path.join(THEMESPATH, self.theme,'core_voldown_button_ro.png')
        px, py = pos
        
        prev = os.path.join(THEMESPATH, self.theme, 'core_volume_button.png')
        prev_ro = os.path.join(THEMESPATH, self.theme, 'core_volume_button_ro.png')
        next = os.path.join(THEMESPATH, self.theme, 'core_volmute_button.png')
        next_ro = os.path.join(THEMESPATH, self.theme, 'core_volmute_button_ro.png')
        
        self.lbl0 = Label(_("Quiz voice"), pos, fsize=18, padding=4, minh=48)
        px += self.lbl0.rect.w + 10
        self.voicetoggle = TransPrevNextButton((px, py), self._cbf_toggle_voice, \
                                  prev, prev_ro,\
                                  next, next_ro, states=[True,False])
        py += self.voicetoggle.rect.h + 20
        px = pos[0]
        self.lbl1 = Label(self.volstr, pos, fsize=18, padding=4, border=1, minh=48)
        self.volumetoggle = TransPrevNextButton((px, py), self._cbf_toggle_volume, \
                                  prev, prev_ro,\
                                  next, next_ro)
        px += self.volumetoggle.rect.w + 20
        self.voldownbut = TransImgButton(imgdown, imgdown_ro, (px, py))
        self.voldownbut.mouse_hover_leave_action = True
        self.voldownbut.connect_callback(self._cbf, MOUSEBUTTONUP, -5)
        
        px += self.voldownbut.rect.w
        self.lbl1.moveto((px , py+4))
        px += self.lbl1.rect.w
        
        self.volupbut = TransImgButton(imgup, imgup_ro, (px, py))
        self.volupbut.mouse_hover_leave_action = True
        self.volupbut.connect_callback(self._cbf, MOUSEBUTTONUP, 5)
        if self.volume == 0:
            self.volumetoggle.toggle()
            self.volumetoggle.display_sprite()
        if not voice_unmute:
            self.voicetoggle.toggle()
            self.voicetoggle.display_sprite()
            
    def set_use_current_background(self, bool):
        self.voldownbut.set_use_current_background(bool)
        self.volupbut.set_use_current_background(bool)
        self.volumetoggle.but.set_use_current_background(bool)
        self.voicetoggle.but.set_use_current_background(bool)
        
    def _cbf_toggle_volume(self, widget, event, data):
        self.logger.debug("set volume %s" % data)
        if self.volume > 0:
            self.volume = 0
        else:
            self.volume = 75
        self.volumetoggle.toggle()
        self.volumetoggle.display_sprite()
        self.volstr = '%02d' % self.volume + "%"
        self.lbl1.erase_sprite()
        self.lbl1.settext(self.volstr)
        self.lbl1.display_sprite()
        subprocess.Popen("amixer set Master %s" % self.volume + "%",shell=True)

    def _cbf_toggle_voice(self, *args):
        self.voicetoggle.toggle()
        self.voicetoggle.display_sprite()
        
    def _cbf(self, widget, event, data):
        if self.volume >= 100 and data[0] > 0:
            return
        if self.volume <= 0 and data[0] < 0:
            return
        self.volume += data[0]
        if self.volume > 99:
            self.volume = 99
        if self.volume < 0:
            self.volume = 0
        if self.volume > 0 and self.volumetoggle.get_state() == 'next':
            self.volumetoggle.toggle()
            self.volumetoggle.display_sprite()
        self.volstr = '%02d' % self.volume + "%"
        self.lbl1.erase_sprite()
        self.lbl1.settext(self.volstr)
        self.lbl1.display_sprite()
        self.logger.debug("Volume set to %s" % self.volstr)
        #Here we set the actual volume (We do this with aumix)
        if self.WEHAVEAUMIX:
            subprocess.Popen("amixer set Master %s" % self.volume + "%",shell=True)
            self.soundcheck.play()
        else:
            self.logger.warning("Not possible to set volume level")
        
    def display(self):
        self.voldownbut.display_sprite()
        self.volupbut.display_sprite()
        self.lbl1.display_sprite()
        self.lbl0.display_sprite()
        self.volumetoggle.display_sprite()
        self.voicetoggle.display_sprite()
    
    def hide(self):
        self.voldownbut.erase_sprite()
        self.volupbut.erase_sprite()
        self.lbl1.erase_sprite()
        self.lbl0.display_sprite()
        self.volumetoggle.erase_sprite()
        self.voicetoggle.erase_sprite()
    
    def get_actives(self):
        return [self.voldownbut, self.volupbut, self.lbl0,  self.lbl1,\
                self.volumetoggle.get_actives(),self.voicetoggle.get_actives()]
    
    def get_volume(self):
        return int(self.volume)

    def get_voice_state(self):
        return self.voicetoggle.get_state()
            
class ScrollWindow(Widget):
    """Widget that holds other widgets and scrolls them up or down."""
    def __init__(self, pos, size, objects, padding=4, childpadding=20, \
                 scrollstep=0, border=0, cols=0, scrollbutton_size='64', \
                 bordercol=BLACK, autoscroll=False):
        """pos - position
        size - size of the scrollwindow
        objects - list with SPWidget object to display
        padding - space between the scrollwindow and the children
        childpadding - space between the children
        scrollstep - amount of pixels to scroll
        border - a border around the window thickness in pixels. 0 means no border
        cols - the amount of columns. 0 means that the amount of columns depends
        on the amount of children on a row. (Currently only 0 or 1 are supported)
        button_up and button_down - Defaults to None and than a text button is used.
        When it's not None a file path is assumed.
        scrollbutton_size - defaults to 64. Supported 
        A special attribute is added to the child called _scroll_focus."""
        # TODO: add append and insert methods
        Widget.__init__(self)
        self.image = pygame.Surface((1,1))# image is needed for a sprite object
        self.theme = self.THEME['theme']
        self.rect = pygame.Rect(pos+size)
        self.border = border
        self.bordercol = bordercol
        self.button_up = os.path.join(self.THEME['themepath'], '%s_scroll_up.png' % scrollbutton_size)
        self.button_up_ro = os.path.join(self.THEME['themepath'], '%s_scroll_up_ro.png' % scrollbutton_size)
        self.button_down = os.path.join(self.THEME['themepath'], '%s_scroll_down.png' % scrollbutton_size)
        self.button_down_ro = os.path.join(self.THEME['themepath'], '%s_scroll_down_ro.png' % scrollbutton_size)
        if self.border:
            self._draw_border()
        self.autoscroll = autoscroll
        self.rect.topleft = pos
        self.visiblerect = self.rect.inflate((-border*2, -border*2))
        start_x = pos[0] + padding
        start_y = pos[1] + padding
        max_x = pos[0] + size[0]
        max_y = pos[1] + size[1]
        x = start_x
        y = start_y
        biggest_width = 1
        firstobject = cols == 1
        _objects = []
        for obj in objects:
            if obj.get_sprite_width() > biggest_width:
                biggest_width = obj.get_sprite_width()
        
        obj_per_row = (size[0] / (biggest_width + childpadding))
        count = 0
        for obj in objects:
            if firstobject:
                y = y - obj.get_sprite_height() - childpadding
                firstobject = False
            if count == obj_per_row or cols == 1:
                x = start_x + padding + childpadding
                y = y + obj.get_sprite_height() + childpadding
                count = 1
            else:
                x += padding + childpadding
                count += 1
            obj.moveto((x, y))
            if self.rect.contains(obj.rect):
                obj._scroll_focus = True
            else:
                obj._scroll_focus = False
            
            x += biggest_width
            _objects.append(obj)
        if _objects:
            self.set_objects(_objects)
            if scrollstep == 0:
                self.scrollstep = objects[0].get_sprite_height() + childpadding
            else:
                self.scrollstep = scrollstep
    
    def _draw_border(self):
        pygame.display.update(pygame.draw.rect(self.screen, self.bordercol, self.rect, self.border))
            
    def set_objects(self, objects):
        self._firstchild = objects[0]
        self._firstchild_start = self._firstchild.get_sprite_pos()
        self._lastchild = objects[-1]
        self._objects = objects
        # setup the scrollbuttons
        if self.button_up:
            # we assume the rest is also set
            self.upbut = TransImgButton(self.button_up, self.button_up_ro, \
                                    (self.rect.right+4, self.rect.top))
            self.downbut = TransImgButton(self.button_down, self.button_down_ro, \
                                    (self.rect.right+4, self.rect.bottom))
        else:
            self.upbut = Button("up", (self.rect.right+4, self.rect.centery))
            self.downbut = Button("down", (self.rect.right+4, self.rect.centery))
        self.downbut.moveto((self.rect.right+4, self.rect.bottom - self.downbut.get_sprite_height()))
        self.upbut.connect_callback(self._cbf, MOUSEBUTTONUP, 'up')
        self.upbut.display_sprite()
        self.downbut.connect_callback(self._cbf, MOUSEBUTTONUP, 'down')
        self.downbut.display_sprite()  
        #self.display_sprite(pos)
        self.display_children()
    
    def _cbf(self, widget, event, data):
        if data[0] == 'down':
            if not self.rect.contains(self._lastchild.rect):
                self.scrollstep = -abs(self.scrollstep)
                self._scroll_children()
        else:
            if not self.rect.contains(self._firstchild.rect):
                self.scrollstep = abs(self.scrollstep)
                self._scroll_children()
        widget.mouse_hover_leave()
        
    def get_actives(self):
        if not self.rect.contains(self._lastchild.rect) or self.autoscroll:
            self.upbut.display_sprite()
            self.downbut.display_sprite()  
            return [self.upbut, self.downbut,self]
        else:
            return [self]


    def display_children(self):
        for obj in self._objects:
            if obj._scroll_focus:
                obj.display_sprite()
            else:
                break
    
    def erase_children(self):
        for obj in self._objects:
            obj.erase_sprite()
                
    def _scroll_children(self):
        oldclip = self.screen.get_clip()
        self.screen.set_clip(self.visiblerect)
        self.erase_children()
        for obj in self._objects:
            if obj._scroll_focus:# displayed object
                obj.moveto((obj.rect.x, obj.rect.y + self.scrollstep))
                if self.rect.contains(obj.rect):
                    obj.display_sprite()
                else:
                    obj._scroll_focus = False
            else:# not displayed
                obj.moveto((obj.rect.x, obj.rect.y + self.scrollstep))
                if self.rect.contains(obj.rect):
                    obj._scroll_focus = True
                    obj.display_sprite()
        self.screen.set_clip(oldclip)
        pygame.display.update()
    
    # Override the methods from SPWidget.SPSprite
    # We use it to redraw the border if we have one
    def display_sprite(self,*args):
        if self.border:
            self._draw_border()
    def erase_sprite(self,*args):
        pass

class ProgressBar(Widget):
    """Class which provides a surface with a progressbar.(400x100)
    """
    ## TODO: the progress bar is full after 95 steps, not 100.
    ## The reason is that it's not possible to split pixels. :-)
    def __init__(self,pos, size, steps=10,\
                 transparent=False, backcol=GREY, barfcol=DEEPSKYBLUE4, barbcol=BLACK):
        """pos is the postion to place this widget
        size is a tuple with the width and height of the widget
        steps is the amount of steps that should be taken to fill the bar, defaults to 10
        The update call returns the number of steps taken, when the bar is full it will return False
        """
        Widget.__init__(self)
        self.end = size[0]
        self.step = self.end / steps
        self.steps = steps
        self.backcol = backcol
        self.barfcol = barfcol
        self.barbcol = barbcol
        
        self.image = pygame.Surface((size[0],size[1]))
        self.image.fill(backcol)
        self.rect = self.image.get_rect()
        pygame.draw.rect(self.image, barbcol, self.rect.inflate(-8,-8),0)
                
        self.barsurf = pygame.Surface((size[0]-8,size[1]-8))
        self.barsurf.fill(barfcol)
        self.barrect =  self.barsurf.get_rect()
        self.barend = self.step        
        self._initbar()
        self.moveto(pos)
                
    def _initbar(self):
        self.x, self.progress = 0, 0 # used in update
        
    def update(self):
        """Update the progressbar with step.
        Returns the number of steps passed.
        """
        if self.progress >= self.steps:
            return False
        self.image.blit(self.barsurf, (4,4), (0, 0, self.x + self.barend, self.barrect.h))
        self.display_sprite()
        #self.refresh_sprite()
        self.progress += 1
        self.barend = (self.progress+1) * self.step
        return self.progress
    
    def reset_bar(self, header=''):
        """This will set the bar to 0."""
        self.erase_sprite()
        self._initbar()
        
    def clearbar(self, screen, backgr):
        """Remove the bar from the screen.
        @ backgr is blitted over @screen.
        This calls pygame.display.update"""
        self.erase_sprite()
    




# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           packid.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.packid.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.packid")

# standard modules you probably need
import os,sys, random

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils

# containers that can be used globally to store stuff
class Img:
    pass
class Snd:
    pass

class PacKid:
    def __init__(self,matrix,startx,starty,speed=24):
        self.speed = speed
        self.pac_r = Img.pac_r
        self.pac_r_c = Img.pac_r_c
        self.pac_l = Img.pac_l
        self.pac_l_c = Img.pac_l_c
        self.pac_u = Img.pac_u
        self.pac_u_c = Img.pac_u_c
        self.pac_d = Img.pac_d
        self.pac_d_c = Img.pac_d_c
        self.pac_smile = Img.pac_smile
        self.img = self.pac_smile
        
        self.rect = self.img.get_rect()
        self.rect.move_ip(startx,starty)
        
        self.startrect = self.img.get_rect()
        self.startrect.move_ip(startx,starty)
        self.matrix = matrix
        self.row = 1
        self.col = 1
        self.dir_dic = {'UP':self._up,'DOWN':self._down,'LEFT':self._left,'RIGHT':self._right}
                
    def update(self,direc):
        Snd.waka.play()
        apply(self.dir_dic[direc])
        return (self.row,self.col)
        #print self.row,self.col,self.rect
       
    def _up(self):
        if self.img == self.pac_u_c:
            self.img = self.pac_u
        else:
            self.img = self.pac_u_c
        if self.matrix[self.row-1][self.col]: # can we go here?
            self.row -= 1
            self.rect.move_ip(0,-self.speed)
        
    def _down(self):
        if self.img == self.pac_d_c:
            self.img = self.pac_d
        else:
            self.img = self.pac_d_c
        if self.matrix[self.row+1][self.col]: # can we go here?
            self.row += 1
            self.rect.move_ip(0,self.speed)
        
    def _left(self):
        if self.img == self.pac_l_c:
            self.img = self.pac_l
        else:
            self.img = self.pac_l_c
        if self.matrix[self.row][self.col-1]: # can we go here?
            self.col -= 1
            self.rect.move_ip(-self.speed,0)
    
    def _right(self):
        if self.img == self.pac_r_c:
            self.img = self.pac_r
        else:
            self.img = self.pac_r_c
        if self.matrix[self.row][self.col+1]: # can we go here?
            self.col += 1
            self.rect.move_ip(self.speed,0)
    
class Memory:
    def __init__(self,points=('1','2','3','4','5')):
        self.memory = {}
        self.stack = [None]* len(points)
        
    def remember(self,(key,item)):
        #print key,item
        self.memory[key] = item
        self.stack.insert(0,key)
        try:
            del self.memory[self.stack.pop()]
        except KeyError:
            pass
            
    def recall(self,key):
        if self.memory.has_key(key):
            return self.memory[key] 

class Letters(Memory):
    instance = 0
    def __init__(self,char,fcol,dest,ttf, y):
        Letters.instance += 2
        self.wait = Letters.instance
        Memory.__init__(self)
#        print "char",char
        self.char = char
        self.image = utils.char2surf(char,16,fcol,ttf, bold=True)
        self.rect = self.image.get_rect()
        self.lifetime = 50 # if we get in a loop
        # calculate the pix pos row*24+60, col*24+150
        self.row = 8
        self.col = 12
        self.rect.move_ip(self.col*24+150,self.row*24+60+y)
        self.dest = dest
        
        self.old_move = None
        #self.org = (self.row,self.col)
        #self.vector = self.dest[0]-self.org[0],self.dest[1]-self.org[1]
        self.matrix = Img.matrix
        self.speed = 24

    def update(self):
        if self.wait:
            self.wait -= 1
            return 0
        self.lifetime -= 1
        if self.lifetime < 0:
            #print 'died'
            return 1 # he died :-(
        self.org = (self.row,self.col)
        stop = self._iq((self.dest[0]-self.org[0],self.dest[1]-self.org[1]))
        return stop
            
    def _iq(self,vector):
        direcs = self._where_to_go()
        if (self.row,self.col) == (8,12):# still in the box
            direcs = ['u']
        if len(direcs) == 1:
            self._move(direcs[0])
            stop = self.dest == (self.row,self.col)
            return stop
        else:
            try:
                direcs.remove(self.old_move)
            except ValueError:
                pass
        if len(direcs) == 1:
            self._move(direcs[0])
            stop = self.dest == (self.row,self.col)
            return stop
        else:
            prior = [None]*4
            #print vector
            if abs(vector[0]) > abs(vector[1]): # y is pref.
                if vector[0] < 0:
                    prior[0] = 'u'
                    prior[3] = 'd'
                else:
                    prior[0] = 'd'
                    prior[3] = 'u'
                
                if vector[1] < 0:
                    prior[1] = 'l'
                    prior[2] = 'r'
                else:
                    prior[1] = 'r'
                    prior[2] = 'l'
                
            else:
                if vector[1] < 0:
                    prior[0] = 'l'
                    prior[3] = 'r'
                else:
                    prior[0] = 'r'
                    prior[3] = 'l'
                if vector[0] < 0:
                    prior[1] = 'u'
                    prior[2] = 'd'
                else:
                    prior[1] = 'd'
                    prior[2] = 'u'
        
        #print 'direcs',direcs,'prior',prior,'row/col',self.row,self.col
        for d in prior:
            if d in direcs:
                if self.recall((self.row,self.col)) == d:
                    #print self.recall((self.row,self.col))
                    d = random.choice(direcs)
                    self._move(d)
                    break
                else:
                    #print d
                    self.remember(((self.row,self.col),d))
                    self._move(d)
                    break
        stop = self.dest == (self.row,self.col)
        #print 'stop-iq',stop
        return stop
        
    def _where_to_go(self):
        direcs = []
        if self.matrix[self.row-1][self.col]: # up
            direcs.append(('u'))
        if self.matrix[self.row+1][self.col]: # down
            direcs.append(('d'))
        if self.matrix[self.row][self.col-1]: # left
            direcs.append(('l'))
        if self.matrix[self.row][self.col+1]: # right
            direcs.append(('r'))
        return direcs
        
    def _move(self,choice):
        
        if choice == 'u':
            self.row -= 1
            self.rect.move_ip(0,-self.speed)
            self.old_move = 'd'
        elif choice == 'd':
            self.row += 1
            self.rect.move_ip(0,self.speed)
            self.old_move = 'u'
        elif choice == 'l':
            self.col -= 1
            self.rect.move_ip(-self.speed,0)
            self.old_move = 'r'
        elif choice == 'r':
            self.col += 1
            self.rect.move_ip(self.speed,0)
            self.old_move = 'l'
        return        

class Word:
    """  Highlight the letter in the word"""
    def __init__(self,word,ttf,cpg):
        self.chars = word
        self.ttf = ttf
        self.index = -1
        self.fontsize = 12
        self.cpg = cpg

        self.fcol = (255,255,51)
        self.hfcol = (255,0,0)

    def update(self):
        if self.index < len(self.chars):
             self.index += 1
        else:
            return self.surf
        return self._render(),self.chars[self.index]

    def _render(self):
        #print self.index
        high_letter,lettersize = utils.text2surf(self.chars[self.index],self.fontsize,self.hfcol,self.ttf,1, bold=True)

        if self.index == 0: # Highlight first letter
            split = self.chars[1:]
            word,wordsize = utils.text2surf(split,self.fontsize,self.fcol,self.ttf,1, bold=True)
            surfsize = (lettersize[0]+wordsize[0],lettersize[1])
            surf = pygame.Surface(surfsize).convert()
            if self.cpg.get_localesetting()[1]: # RTL
                surf.blit(word,(0,0))
                surf.blit(high_letter,(wordsize[0],0))
            else:
                surf.blit(high_letter,(0,0))
                surf.blit(word,(lettersize[0],0))

        elif self.index == len(self.chars)-1: # Highlight last letter
            split = self.chars[:-1]
            word,wordsize = utils.text2surf(split,self.fontsize,self.fcol,self.ttf,1, bold=True)
            surfsize = (lettersize[0]+wordsize[0],lettersize[1])
            surf = pygame.Surface(surfsize).convert()
            if self.cpg.get_localesetting()[1]: # RTL
                surf.blit(high_letter,(0,0))
                surf.blit(word,(lettersize[0],0))
            else:
                surf.blit(word,(0,0))
                surf.blit(high_letter,(wordsize[0],0))

        else: # Letter inside the word, the word must be split in two parts
            word,wordsize = utils.text2surf(self.chars[:self.index],self.fontsize,\
                            self.fcol,self.ttf,1, bold=True)
            word1,word1size = utils.text2surf(self.chars[self.index+1:],self.fontsize,\
                              self.fcol,self.ttf,1, bold=True)
            surfsize = (lettersize[0]+wordsize[0]+word1size[0],lettersize[1])
            surf = pygame.Surface(surfsize).convert()
            if self.cpg.get_localesetting()[1]: # RTL
                surf.blit(word1,(0,0))
                surf.blit(high_letter,(word1size[0],0))
                surf.blit(word,(word1size[0]+lettersize[0],0))
            else:
                surf.blit(word,(0,0))
                surf.blit(high_letter,(wordsize[0],0))
                surf.blit(word1,(lettersize[0]+wordsize[0],0))
        self.surf = surf # see self.update
        return self.surf

class SidePanel:
    def __init__(self,levels,ttf,blitpos):
        self.ttf = ttf
        self.surf = pygame.Surface((130,400)).convert()
        pygame.draw.rect(self.surf,(42,191,44),self.surf.get_rect(),6)
        self.when = 9# number of gooditems to play last level
        self.level_pos_list = [(10,10),(10,110),(10,210),(10,310)]
        self.inlevel_word_pos_list = [(26,10),(26,110),(26,210),(26,310)]
        
        self.x = 30
        self.y = 6
        self.fsize = 12
        
        self.fcol = (255,255,51)
        i = 1
        for l in levels[:-1]: # The last level is different
            self._set_level("Level "+str(l+1)+":",i)
            i += 1
        self.lastlevel_y = self.level_pos_list[i-1]
        self.fcol = (186,184,154)
        self._set_last_level(self.level_pos_list[3])
        self.fcol = (255,255,51)
        #set x and y to the first position
        self.y = self.level_pos_list[0][1]
        self.x = self.level_pos_list[0][0]+16
        
        self.goodimg = pygame.transform.scale(Img.pac_smile,(12,12))
        self.wrongimg = pygame.transform.scale(Img.pac_sad,(12,12))
        self.gooditems = 0
        self.count,self.level = 0,0

        self.switch_task()
               
    def _set_last_level(self,pos):
        self._set_level("Level "+str(4)+":", 4)
    
    def _set_level(self,levelstr,level):
        pos = self.level_pos_list[level-1]
        levelimg = utils.char2surf(levelstr,self.fsize+2,self.fcol,self.ttf, \
                             antialias=True, bold=True)
        self.surf.blit(levelimg,pos)
                             
    def set_word(self,word):
        wordimg, size = utils.text2surf(word,self.fsize-2,self.fcol,self.ttf,sizel=True,\
                                   antialias=True, bold=True)
        self.y += size[1]
        self.surf.blit(wordimg,(self.x,self.y))
            
    def switch_task(self):
        SwitchLevel = False
        if self.count == 3:
            self.level += 1
            self.y = self.inlevel_word_pos_list[self.level][1]
            self.count = 0
            SwitchLevel = True
        self.count += 1
        return SwitchLevel
    
    def set_good_wrong(self,gw):
        if gw > 2:
            self.surf.blit(self.wrongimg,(self.x-16,self.y+4))
        else:
            self.surf.blit(self.goodimg,(self.x-16,self.y+4))
            self.gooditems += 1
    
    def get_surf(self):
        return self.surf.convert()
        
    def get_level_score(self):
        #print 'gooditems',self.gooditems
        if self.gooditems >= self.when:
            self.when = 0
            return 1
        else:
            return 0

class LastLevelImg:
    def __init__(self,music,img,pos):
        self.char = 'FRUIT' # to be compatible with the letters class in the loop
        self.file = os.path.join(Img.datadir,music)
        self.img = utils.load_image(os.path.join(Img.datadir,img),1)
        self.pos = pos
        self.rect = self.img.get_rect()
        # calculate the pix pos row*24+60, col*24+150
        self.row,self.col = pos
        self.rect.move_ip(self.col*24+150,self.row*24+60)
        
    def eat(self):
        utils.load_music(self.file).play()
        return 25 #score


class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.packid.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','PackidData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'c'))
        self.rchash['theme'] = self.theme
        self.logger.debug("found rc: %s" % self.rchash)
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        
        self.language = self.SPG.get_localesetting()[0]
        self.ttf = None
        self.stopflag = None# used to stop the loop when not level 4
        self.score = 0
        self.letters_spots = []
        self.gamelevels = range(4)
        self.oldvolume = 0
    
    def __del__(self):
        #print 'Reached del'
        try:
            Snd.walk.stop()
        except:
            pass  
    
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.refresh()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Packid")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "packid"
    
    def get_help(self):
        """Mandatory methods"""
        text=[_("The aim of the game:"),
        _("Try to 'eat' all the letters in the appropriated order."),
        _("There are thee levels with three words each."),
        _("When you have finished all the levels, without making to much mistakes"),
        _("(max two per word), you can play the last level which is a maze."),
        _("Try to find the way out while eating the fruits for extra points and"),
        _("funny sounds :-)")]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Fun/Alphabet")
        
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 4
    
    def pre_level(self,level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        self.logger.debug("pre_level called with: %s" % level)
        pass
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("nextlevel called with: %s" % level)
        if level == 5:
            return False
        self.level = level
        self.dbmapper = dbmapper
        # Your top blit position, this depends on the menubar position 
        if self.blit_pos[1] == 0:
            y = 10
            self.gamearea = pygame.Rect(0, y, 800, 490) 
        else:
            y = 110
            self.gamearea = pygame.Rect(0, y, 800, 600) 
        return True
    
    def start(self):
        """Mandatory method."""
        Snd.waka = utils.load_music(os.path.join(self.my_datadir,'waka.wav'))
        Snd.walk = utils.load_sound(os.path.join(self.my_datadir,'walk.wav'))
        Snd.finlevel = utils.load_sound(os.path.join(self.my_datadir,'finlevel.wav'))
        Snd.bummer = utils.load_sound(os.path.join(self.CPdatadir,'bummer.wav'))
        Snd.eat = utils.load_sound(os.path.join(self.my_datadir,'eat.wav'))
        
        Img.datadir = self.my_datadir

        Img.pac_u = utils.load_image(os.path.join(self.my_datadir,'pac_u.png'),1)
        Img.pac_u_c = utils.load_image(os.path.join(self.my_datadir,'pac_u_c.png'),1)
        Img.pac_d = utils.load_image(os.path.join(self.my_datadir,'pac_d.png'),1)
        Img.pac_d_c = utils.load_image(os.path.join(self.my_datadir,'pac_d_c.png'),1)
        Img.pac_l = utils.load_image(os.path.join(self.my_datadir,'pac_l.png'),1)
        Img.pac_l_c = utils.load_image(os.path.join(self.my_datadir,'pac_l_c.png'),1)
        Img.pac_r = utils.load_image(os.path.join(self.my_datadir,'pac_r.png'),1)
        Img.pac_r_c = utils.load_image(os.path.join(self.my_datadir,'pac_r_c.png'),1)
        Img.pac_smile = utils.load_image(os.path.join(self.my_datadir,'pac_smile.png'),0)
        Img.pac_sad = utils.load_image(os.path.join(self.my_datadir,'pac_sad.png'),0)
        
        self.sidepan = SidePanel(self.gamelevels,self.ttf,self.blit_pos)
        
        # test if we have a words list for the locale
        try:
            wordsloc = self.SPG.get_localesetting()[0].split('_')[0]
            wordlist = 'words-'+wordsloc
            if not os.path.exists(os.path.join(self.my_datadir,wordlist)):
                raise IndexError
        except IndexError,info:
            self.logger.info("Can't find words for locale %s Using english for the words and sounds." % wordsloc)
            wordlist = 'words-en'
            wordsloc = 'en'
        try:
            items = utils.read_unicode_file(os.path.join(self.my_datadir,wordlist))
        except IOError,info:
            self.logger.exception("can't open or read words file %s" % \
                                  os.path.join(self.my_datadir,wordlist))
            raise MyError
        if self.SPG.get_localesetting()[1]:
            self.words = [s[:-1] for s in items]
        else:
            self.words = [s[:-1].upper() for s in items]
        #print self.words
        # Look for alphabet sounds for this locale.
        # If all fails we fall back to the "old" wahoo sound.
        self.alphabetdir = self.SPG.get_absdir_loc_path()
        if self.alphabetdir == 'en' and self.alphabetdir not in self.language:
            # if true the sound dir wasn't found and 'en' is returnt as fallback
            self.alphabetdir = None
        self.SPG.tellcore_set_dice_minimal_level(3)
        
        
    def _setup_last_level(self):
        self.fruits,spots = [],[]
        self.letters_spots = []# use this again for the fruits
        music = ['pac1.ogg','pac2.ogg','pac3.ogg','pac4.ogg','pac5.ogg','pac6.ogg']
        random.shuffle(music)
        for file in ('kers.png','banaan.png','aardbei.png','citroen.png','appel.png','peer.png'):
            spot = self._rand_spot()
            if spot in spots or spot in ((1,1),(15,23),(15,24),(16,23),(16,24)):
                spot = self._rand_spot()
            spots.append((spot))
            self.letters_spots.append((spot))
            self.fruits.append((LastLevelImg(music.pop(),file,spot)))
        self.letters_spots.append(((15,23)))    
    
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        self.next_exercise()
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        # As this is called when a level is finished we also use it to store
        # dbase stuff.
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
        
        self.dbmapper.insert('word',self.word)
        self.dbmapper.insert('wrong',self.gw)
        
        score = max(1,(10-self.gw) - (seconds/150.0))
        self.logger.debug("score is: %s" % score)
        return score
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        if self.stopflag:
            self.SPG.tellcore_game_end(True)
        self.stop = 0
        self.score = 0
        for event in events:
            pos = pygame.Rect(pygame.mouse.get_pos() + (4,4))
            if event.type is KEYDOWN and not self.objs_to_move:
                key = self._on_keypress(event.key)
                if key:
                    r = self.screen.blit(Img.backgr,self.packid.rect.inflate(2,2),self.packid.rect.inflate(2,2))
                    self.pack_pos = self.packid.update(key)
                    rr = self.screen.blit(self.packid.img,self.packid.rect)
                    pygame.display.update((r,rr))
        if self.objs_to_move: # Are there letters to move?
            self._move_objs()
        
        elif self.pack_pos in self.letters_spots: # Have we hit a letter?
            if self.level == 4 and self.pack_pos == (15,23):# pac hit exit in last level
                self.SPG.tellcore_game_end(True)
            else:
                for item in self.letter_objs:
                    if self.pack_pos == (item.row,item.col):
                        if item.char == 'FRUIT':
                            self.score = item.eat()
                            self.letter_objs.remove(item)
                            pygame.display.update(self.screen.blit(Img.backgr_start,item.rect,item.rect))# erase fruit
                            self.packid.img = Img.pac_smile
                            pygame.display.update(self.screen.blit(Img.pac_smile,self.packid.rect))
                        else:
                            self._check_letter(item)
                        break
        return 
    def next_exercise(self): 
        pygame.display.update(self.screen.fill((0,0,0), self.gamearea))
        self.letters_spots = []
        if self.level == 1:
            self.brick = utils.load_image(os.path.join(self.my_datadir,'brick.png'),0)
            matrix = self._read_grid('grid0.txt')
            self._build_field(matrix)
        elif self.level == 2:
            self.brick = utils.load_image(os.path.join(self.my_datadir,'leafs.png'),0)
            matrix = self._read_grid('grid1.txt')
            self._build_field(matrix,1)
        elif self.level == 3:
            self.brick = utils.load_image(os.path.join(self.my_datadir,'sea.png'),0)
            matrix = self._read_grid('grid2.txt')
            self._build_field(matrix,1)
        elif self.level == 4:
            if self.sidepan.get_level_score():
                load_music(os.path.join(self.my_datadir,'feelgood.ogg')).play()
                
                self.brick = utils.load_image(os.path.join(self.my_datadir,'camo.png'),0)
                m = MazeGen(17,25)
                matrix = m.get_maze()
                
                #matrix = self._read_grid('grid3.txt')
                self._build_field(matrix)
                self._setup_last_level()
            else:
                text = _("The last level is only available when you finish the first three levels without errors.")
                self.SPG.tellcore_info_dialog(text)
                self.SPG.tellcore_game_end(store_db=True)
                return
        else:
            self.stopflag = 1
            return
                       
        Img.backgr = self.screen.convert()
        Img.backgr_start = self.screen.convert()# keep a begin situation surface
        
        line1 = _("Find all the letters in the right order.")
        line2 = _("The word to find is: ")
        ### TODO fix RTL issues here
        if self.level != 4:
            if self.SPG.get_localesetting()[1]:
                surf,spam = utils.text2surf(line1,12,(183,255,50),self.ttf,1, bold=True)
                pygame.display.update(self.screen.blit(surf,(780-spam[0],0+self.gamearea.top)))
                surf,self.surfword_offset = utils.text2surf(line2,12,(183,255,50),self.ttf,1, bold=True)
                pygame.display.update(self.screen.blit(surf,(780-self.surfword_offset[0],24+self.gamearea.top)))
                word = random.choice(self.words)
            else:
                surf,spam = utils.text2surf(line1,12,(183,255,50),self.ttf,1, bold=True)
                pygame.display.update(self.screen.blit(surf,(28,4+self.gamearea.top)))
                surf,self.surfword_offset = utils.text2surf(line2,12,(183,255,50),self.ttf,1, bold=True)
                pygame.display.update(self.screen.blit(surf,(8,30+self.gamearea.top)))
                word = random.choice(self.words)
        
        # start the show and set some initial values
        self.gw = 0 #good/wrong counter
        if self.level == 4:
            self.word = ''
            self.objs_to_move,self.letter_objs = [],[]
            for obj in self.fruits:
                pygame.display.update(self.screen.blit(obj.img,obj.rect))
                self.letter_objs.append((obj))
                r = self.screen.blit(self.packid.img,(174,84))# blit packid
                pygame.display.update(r)
        else:
            self.word = word
            self._start_letters(word)
            self.sidepan.set_word(word)
            self._update_sidepanel()
            self.surfword = Word(word,self.ttf,self.SPG)
            self._update_surfword()
        x = 174
        y = 84 + self.gamearea.top
        self.packid = PacKid(matrix,x,y,24)
        
        self.pack_pos = (self.packid.row,self.packid.col)# start postion
        if self.level < 4:
            self.oldvolume = Snd.walk.get_volume()
            #print "volume:",self.oldvolume
            Snd.walk.set_volume(0.4)
            Snd.walk.play(-1)# stop when the letters are stopped (in def _move_objs)
        return True
    
    def _update_surfword(self):
        surf, letter_to_speak = self.surfword.update()
#        if self.SPG.get_localesetting()[1]:
#            # The Hebrew chars are rendered with a lot of empty space
#            # So we cut of the top and bottom to get it to blit in a mal space
#            cutoff_rect = surf.get_rect().inflate(0,-10)
#            pygame.display.update(self.screen.blit(surf,\
#                    (780-self.surfword_offset[0]-65,30),cutoff_rect))
#        else:
        pygame.display.update(self.screen.blit(surf,\
                        (8+self.surfword_offset[0],30+self.gamearea.top)))
        pygame.time.wait(1000)
    
    def _speak_letter(self,letter):
        """Plays the alphabet soundfile for @letter
        Return True on succes and False on failure"""
        return utils.speak_letter(letter.lower(), self.language)
           
    def _update_sidepanel(self):
        pygame.display.update(self.screen.blit(self.sidepan.get_surf(),(0,60 + self.gamearea.top)))
    
    def _read_grid(self,name):
        grid = []
        filename = os.path.join(self.my_datadir,name)
        f = open(filename,'r')
        for line in f.readlines():
            grid.append((tuple(map(int,line[:-1]))))
        f.close()
        return tuple(grid)
    
    def _build_field(self,matrix, ran=0):
        Img.matrix = matrix #stash ref
        surf = pygame.Surface((620,428))
        y = 10
        for row in matrix:
            x = 10
            for item in row:
                if not item:
                    if ran:
                        brick = self.brick#.convert()
                        angle = random.choice((0,90,180,270))
                        surf.blit(pygame.transform.rotate(brick,angle),(x,y))
                    else:
                        surf.blit(self.brick,(x,y))
                elif item == 2:# special
                    i = utils.load_image(os.path.join(self.my_datadir,'exit.png'),0)
                    #image = 48x48
                    #x -= 24
                    #y -= 24
                    surf.blit(i,(x,y))
                    break
                x +=24
            y += 24
        pygame.display.update(self.screen.blit(surf,(140,50 + self.gamearea.top)))
        
    def _start_letters(self,word):
        self.word,spots = word,[]
        self.letter_to_find = 0
        self.objs_to_move,self.letter_objs = [],[]# lists to hold Letters objects
        Letters.instance = 0 #reset instance counter
        for item in word:
            spot = self._rand_spot()
            if spot in spots or spot in ((8,11),(8,12),(8,13),(1,1)):# same coords or inside the box,packid
                spot = self._rand_spot()
            spots.append((spot))
            self.objs_to_move.append((Letters(item,(255,255,255),spot,self.ttf, self.gamearea.top)))
            
            #print item,spot,Img.matrix[spot[0]][spot[1]]

    def _rand_spot(self):
        #return (1,3)
        while 1:
            spot = (random.randrange(1,17),random.randrange(1,24))
            if Img.matrix[spot[0]][spot[1]]:
                break
        return spot
        
    def _on_keypress(self,key):
        if key == K_UP:
            return 'UP'
        elif key == K_DOWN:
            return 'DOWN'
        elif key == K_LEFT:
            return 'LEFT'
        elif key == K_RIGHT:
            return 'RIGHT'
        
    def _move_objs(self):
        objs_to_remove,dirty_rects = [],[]
        for obj in self.objs_to_move:
            dirty_rects.append((self.screen.blit(Img.backgr,obj.rect.inflate(2,2),obj.rect.inflate(2,2))))
            flag = obj.update()
            if flag:
                rec = obj.rect
                rec.move_ip(4,0)
                Img.backgr.blit(obj.image,rec)
                objs_to_remove.append((obj))
                self.letter_objs.append((obj)) # store letters objs 
                # store the pos of the letter, used to check collide with packid
                # take these positions because a obj maybe died (look in class)
                self.letters_spots.append(((obj.row,obj.col))) 
            dirty_rects.append((self.screen.blit(obj.image,obj.rect)))
        pygame.time.wait(100)
        pygame.display.update(dirty_rects)
        
        if objs_to_remove:
            for item in objs_to_remove:
                self.objs_to_move.remove(item)
                if self.objs_to_move == []:
                    Snd.walk.stop()
                    Snd.walk.set_volume(self.oldvolume)
                    r= self.screen.blit(self.packid.img,self.packid.rect)# blit packid
                    pygame.display.update(r)
                    pygame.time.wait(500)
    
    def _check_letter(self,obj):
        if obj.char == self.word[self.letter_to_find]:
            # Good letter
            #print 'letter',obj.char
            self.letter_objs.remove(obj)
            Img.backgr.blit(Img.backgr_start,obj.rect.inflate(8, 8),obj.rect.inflate(8, 8))# erase letter
            self.packid.img = Img.pac_smile
            pygame.display.update(self.screen.blit(Img.pac_smile,self.packid.rect))
            if self.alphabetdir:
                if not self._speak_letter(obj.char):
                    Snd.eat.play()
            else:
                Snd.eat.play()
            pygame.time.wait(500)
            
            self.score = 10
            self.letters_spots.remove(self.pack_pos)
            #print self.letters_spots
            if self.letters_spots:
                self.letter_to_find += 1 
                self._update_surfword()
            else: # last letter 
                Snd.finlevel.play()
                self.score = 100
                Img.backgr.blit(Img.backgr_start,obj.rect,obj.rect)# erase letter
                self.sidepan.set_good_wrong(self.gw)
                self._update_sidepanel()   
                pygame.time.wait(2000)
                if self.sidepan.switch_task():
                    self.SPG.tellcore_level_end(store_db=True)
                else:
                    self.next_exercise() 
        else:
            #wrong letter
            if self.packid.img is Img.pac_sad:
                return # still on the wrong letter
            self.gw += 1
            Snd.bummer.play()
            self.packid.img = Img.pac_sad
            pygame.display.update(self.screen.blit(Img.pac_sad,self.packid.rect))
            pygame.time.wait(500)
        



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           fishtank.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.fishtank.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.fishtank")

# standard modules you probably need
import os,sys
import glob,random
import pygame
from pygame.constants import *

import utils
import Timer
from SPConstants import *
import SPSpriteUtils

# containers that can be used globally to store stuff
class Misc:
    # fish_clear_total is used as a counter.
    # Fish objects add the number of update loops they do before the user clicks
    # them. In the end we have a number which represents the number of update loops
    # it took to clear all the fish.
    # The lower this number is the quicker the user was in removing the fish.
    # By putting it into a class namespace all the objects can use it.
    fish_clear_total = 0
    # used by the scoredisplay
    score = 0

class Bubble(SPSpriteUtils.SPSprite):
    def __init__(self,image,snd, top):
        """Bubble that starts at the bottom and floats to the surface.
        It also plays a 'blub' sound"""
        self.image = image
        #SPSpriteUtils.SPSprite will set self.rect
        SPSpriteUtils.SPSprite.__init__(self,self.image)
        # we don't care about the iterator returnt as we let SPSprite call the 
        # SPSprite.on_update method. We do set the 'dokill' argument as we don't
        # want to reuse the sprite.
        # (see API docs for more info about the SPSprite.on_update method)
        start = (random.randint(50,750),500-self.image.get_height())
        end = (start[0],top)
        self.set_movement(start,end,step=6,retval=0,loop=1,dokill=1)
        snd.play()

class Fish(SPSpriteUtils.SPSprite):
    def __init__(self,imagepair, obs,  parent):
        """@x and @y are the start coords. 
        The fish always starts of screen either right or left.
        @step is the amount of pixels we move
        @delay is used as a counter to delay the movement,
        when it's zero the sprite moveit method is called.
        """
        self.logger = logging.getLogger("childsplay.fishtank.Fish")
        # we will use the two images to create a simple animation in on_update
        self.image_0,self.image_1 = imagepair
        self.image = self.image_0
        self.org_images = (self.image_0.convert_alpha(),self.image_1.convert_alpha())
        SPSpriteUtils.SPSprite.__init__(self,self.image)
        # rects are the same for both images so we don't need to swap rects
        self.rect = self.image.get_rect()
        self.obs = obs
        self.parent = parent
        
    def start(self,x,y,step,delay,react):
        """Set the sprite and start movement"""
        self.update_counter = 0
        self.delay = delay
        self.start = (x,y)# we use this as an end pos in level 6, see callback
        self.step = step
        self.delay = delay
        self.wait = delay
        self.react = react
        # determine the end position (we move horizontale so we don't care about y)
        if x < 1:
            # we start from the left
            endx = 800
        else:
            endx = 0 - self.image.get_width()
        end = (endx,y)# move of the screen
        self.end = end
        loop = 1
        self.moveit = self.set_movement(self.start,end,step,0,loop)# setup the movement of this object
        self.connect_callback(self.callback, event_type=MOUSEBUTTONDOWN)
    
    def mirror_image(self):
        self.image = pygame.transform.flip(self.image,1,0)
        self.image_0 = pygame.transform.flip(self.image_0,1,0)
        self.image_1 = pygame.transform.flip(self.image_1,1,0)

    def on_update(self,*args):
        """This is called by group.refresh method"""
        self.update_counter += 1
        if self.wait:# extra delay, otherwise they move to fast 
            self.wait -= 1
            return
        else:
            self.wait = self.delay
            if self.image == self.image_1:
                self.image = self.image_0
            else:
                self.image = self.image_1
            status = self.moveit.next()
            if status != -1:# reached the end of the movement
                if self.parent.are_we_in_aquarium_mode():
                    # special behaviour, fish turns each time it reaches the end
                    self.mirror_image()
                    self.stop_movement(now=1)
                    start = (self.end[0], self.end[1])
                    end = (self.start[0],self.start[1])
                    self.start = (start[0], start[1])
                    self.end = (end[0],end[1])
                    self.wait = self.delay
                    step = self.step
                    self.moveit = self.set_movement(start,end,step,0)
                else:
                    self.remove_sprite()
                # placeholder: no score

    def callback(self,*args):
        """ This is called when the sprite class update or refresh method is called
        and the mouse event occurs in this sprite.
        Remember that you must 'connect' a callback first with a call to
        self.connect_callback, see the SPSprite reference for info on possibilities
        of connecting callbacks.
        """
        # enable logger to see the args
        #self.logger.debug("callback called with %s, %s" % (args[0],args[1]))
        # when the user hits us this method is called.
        if self.react == 1:
            # special behaviour, fish turns and tries to get away
            self.mirror_image()
            self.stop_movement(now=1)
            start = self.rect.topleft
            end = self.start
            step = self.step*2
            self.moveit = self.set_movement(start,end,step,0,dokill=1)
            self.react += 1
        else:
            self.remove_sprite()# it will not destroyed
            # You must stop the movement because moveit is a generator
            self.stop_movement(now=1)
            # add the number to the global counter
            Misc.fish_clear_total += self.update_counter
            return self.obs((1000 - self.rect.topleft[0]) / 100)

class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.fishtank.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','FishtankData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'fishtank.rc'))
        self.rchash['theme'] = self.theme
        
        self.sounddir = os.path.join(self.my_datadir,'sounds')
        self.bubimg = utils.load_image(os.path.join(self.my_datadir,'backgrounds',self.theme,'blub0.png'))
        self.bubsnd = utils.load_sound(os.path.join(self.sounddir,'blub0.wav'))
        self.splashsnd = utils.load_sound(os.path.join(self.sounddir,'poolsplash.wav'))
        self.WeAreAquarium = False
        self.aquarium_counter = 0
        
        self.aquarium_music_list = glob.glob(os.path.join(self.sounddir,'*.ogg'))
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        self.actives.set_onematch(True)
            
    def clear_screen(self):
        self.screen.set_clip()
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def are_we_in_aquarium_mode(self):
        return self.WeAreAquarium

    def get_moviepath(self):
        movie = os.path.join(self.my_datadir,'help.avi')
        return movie

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.redraw()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Fishtank")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "fishtank"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
        _("Try to remove the fish by clicking on them with the mouse."),
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Fun/Mousetraining")
    
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6

    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and cache the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer_f.stop()
        except:
            pass
        try:
            self.timer_b.stop()
        except:
            pass
        try:
            self.aquariumsound.stop()
        except:
            pass
        self.clear_screen()

    def score_observer(self, score):
        self.aquarium_counter = 0
        self.WeAreAquarium = False
        self.aquarium_text.erase_sprite()
        self.actives.remove(self.aquarium_text)
#        if self.aquariumsound:
#            self.aquariumsound.stop()

        snd = utils.load_sound(os.path.join(self.my_datadir,'sounds', 'poolsplash.wav')) 
        snd.play()
        if score < 0:
            score = 1
        self.scoredisplay.increase_score(score + self.level*6)
    
    def start(self):
        """Mandatory method."""
        # here we setup our list with images we use to construct sprites through out the levels.
        rawimagelist = []
        filelist = glob.glob(os.path.join(self.my_datadir,'*.png'))
        filelist.sort()
        for i in range(0,len(filelist),2):
            rawimagelist.append((filelist[i],filelist[i+1]))
        self.imagelist = []
        try:
            for imgfile in rawimagelist:
                self.imagelist.append((utils.load_image(imgfile[0]),\
                                   utils.load_image(imgfile[1])))
        except Exception,info:
            self.logger.exception("Error loading images")
            raise utils.MyError()
        self.logger.debug("constructed %s groups of images" % len(self.imagelist))
        # get some 'blub' sounds
        self.sounds = []
        filelist = glob.glob(os.path.join(self.my_datadir,'*.wav'))
        for path in filelist:
            self.sounds.append(utils.load_sound(path))
        # level_data: one tuple per level six in total.
        # tuple values are: number of fish,x coord,delay,step,timersleep
        # x coord of 0 means both left and right
        # Third level adjust the speed of some of the fish
        # from the third level the speeds are randomized.
        self.level_data = [(20,-90,8,4,4),(20,0,8,4,4),(20,0,8,4,4),(20,890,4,6,4),\
                            (20,0,4,8,4),(20,0,4,6,4)]

        surf = utils.char2surf("Aquarium mode", 14, fcol=GREEN)
        self.aquarium_text = SPSpriteUtils.MySprite(surf, pos=(30, 580))
        self.aquarium_text.set_use_current_background()
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        # Your top blit position, this depends on the menubar position 
        if self.blit_pos[1] == 0:
            self.ty = 10
        else:
            self.ty = 110
        if level > 6: return False
        self.level = level
        # make sure we don't have a timer running
        self.stop_timer()
        # db_mapper is a Python class object that provides easy access to the 
        # dbase.
        self.db_mapper = dbmapper
        # make sure we don't have fish left in the actives group.
        self.actives.empty()
        self.WeAreAquarium = False
        self.aquarium_counter = 0
        #self.aquariumsound = False
        self.aquariumsound = utils.load_music(random.choice(self.aquarium_music_list))
        #detrmine which background pic we will use:
        backimage = os.path.join(self.my_datadir, 'backgrounds', \
                                 self.theme,'%s.jpg' % level)
        if not os.path.exists(backimage):
            backimage = os.path.join(self.my_datadir, 'backgrounds', \
                                 'childsplay','%s.jpg' % level)
        # remember that SPC sets a clip in relation to the theme used
        self.coral_surf = utils.load_image(backimage)
        self.screen.blit(self.coral_surf,(0,self.ty-10))
        pygame.display.update()
        self.backgr.blit(self.coral_surf,(0,self.ty-10))
        random.shuffle(self.imagelist)
        # get values for the fish objects
        data = self.level_data[level-1]
        startx = data[1]
        delay = data[2]# lower = faster
        step = data[3] # higher = faster
        self.timersleep = data[4] # seconds between fish start
        react = 0
        self.logger.debug("setup fish with: x=%s,delay=%s,step=%s,timer=%s" % \
                                                (startx,delay,step,self.timersleep))
        # setup the fish objects, in two steps
        fishobjects = []
        for item in self.imagelist[:data[0]]:
            # we use copies, this way we can reuse the image list.
            s1, s2 = item[0].convert_alpha(),item[1].convert_alpha()
            # position and position are set when the level is started
            fishobjects.append(Fish((s1,s2), self.score_observer, self))
        
        # setup the objects and put them in a seperate list
        # we don't destroy them so we can use them in any level.
        self.livefish = []
        for obj in fishobjects:
            y = random.randint(self.ty,410+self.ty)
            if startx == 0:
                x = random.choice((-90,890))
            else:
                x = startx
            if x > 0:
                # we move from right to left and we must mirror the original image
                obj.mirror_image()
            if level >= 3:# third level, we adjust some of the speeds
                d = data[2]
                delay = random.choice((d,d,d//2))
                s = data[3]
                step = random.choice((s,s,s*2))
            if level == 6:
                d = data[2]
                delay = d//3
                s = data[3]
                step = s*2
            obj.start(x,y,step,delay,react)
            self.livefish.append(obj)
        
        # we need total fish for score calculation.
        self.totalfish = len(self.livefish)
        # we store the number of fish this level will have.
        self.db_mapper.insert('fish',self.totalfish)
        
        self.aquariumsound.play(-1)
        return True
    
    def pre_level(self, level):
        pass
    
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        # placed here because it's another thread
        self.timer_f = Timer.Timer(self.timersleep,self.release_fish,\
                                   lock=self.SPG.get_thread_lock(), startnow=True)
        self.timer_f.start()
        self.timer_b = Timer.Timer(self.timersleep+1,self.release_bubble,\
                                   lock=self.SPG.get_thread_lock())
        self.timer_b.start() 
        self.screen.set_clip((0, 0, 800, self.ty+490))
    
    def release_fish(self):
        """Called by the timer, and responsible for the swimming of the fish.
        This will get a fish from the livefish list and place it in
        the Sprite actives group, and thereby it will be blitted on the screen."""
        try:
            obj = self.livefish.pop()
        except IndexError:
            return 0# stops the timer
        self.actives.add(obj)
        return 1# let the timer run    

    def release_bubble(self):
        # we don't keep a reference so it will be destroyed by the Python
        # garbage collector when 'kill' is called after the movement is ended
        self.actives.add(Bubble(self.bubimg,self.bubsnd, self.ty))
        return 1# let the timer run   
    
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        #m,s = timespend.split(':')
        #seconds = int(m)*60 + int(s)  
#        c = 0.7
#        score = 10.0 / (max (float(Misc.fish_clear_total/100)/float(self.totalfish), 1.0)) ** c
        score = 8 # fishtank is not really suitable for reall scores, so now we make sure we the core does a levelup
        return score

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        myevent = None
        if self.aquarium_counter > 300 and not self.WeAreAquarium and\
                                     len(self.actives.sprites()):
            self.WeAreAquarium = True
            self.actives.add(self.aquarium_text)
            self.aquarium_text.display_sprite()
#            self.aquariumsound = utils.load_music(random.choice(self.aquarium_music_list))
#            self.aquariumsound.play(-1)
        self.aquarium_counter += 1
        if not self.livefish and not self.actives.sprites():
            # no more sprites left so the level is done.
            self.db_mapper.insert('clearfish',Misc.fish_clear_total)
            # we call the SPGoodies observer to notify the core the level
            # is ended and we want to store the collected data
            levelup = True
            self.SPG.tellcore_level_end(store_db=True, \
                                        level=min(6, self.level + levelup), \
                                            levelup=levelup, no_question=True)
        for event in events:
            if event.type is MOUSEBUTTONDOWN:
                self.aquarium_counter = 0
                self.WeAreAquarium = False
                self.aquarium_text.erase_sprite()
                self.actives.remove(self.aquarium_text)
#                if self.aquariumsound:
#                    self.aquariumsound.stop()
                # no need to query all sprites with all the events
                # we only pass it an event if it's an event were intrested in.
                myevent = event
        # move all fish
        self.actives.refresh(myevent)
        return 
        




# -*- coding: utf-8 -*-

# Copyright (c) 2013 Stas Zytkiewicz stas.zytkiewicz@gmail.com
# Copyright (c) 2013 Ana Lopes anajoaolopes@gmail.com
# Copyright (c) 2013 Helder Costa h24costa@gmail.com
# Copyright (c) 2013 Rui Bento rui.m.bento@gmail.com
#
#           TicTacToe.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation. A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("schoolsplay.TicTacToe.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("schoolsplay.TicTacToe")

# standard modules you probably need
import os,sys, random

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils

def whoPlaysFirst():
    # Who plays first is choosen randomly.
    return random.randint(0, 1)

def drawGrid(screen):
    pygame.draw.line(screen, BLACK, [325, 120], [325,570], 5)
    pygame.draw.line(screen, BLACK, [475, 120], [475,570], 5)
    
    pygame.draw.line(screen, BLACK, [175, 270], [625,270], 5)
    pygame.draw.line(screen, BLACK, [175, 420], [625,420], 5)

    Score(screen)

#0
def GridGame0(screen):
    if player == 1:
        pygame.draw.line(screen, BLUE, [200, 145], [300,245], 10)
        pygame.draw.line(screen, BLUE, [300, 145], [200,245], 10)
    else:
        pygame.draw.circle(screen, RED, [250, 195], 50, 10)
    GridGame[0] = player
    return 1 + player * (-1)

#1
def GridGame1(screen):
    if player == 1:
        pygame.draw.line(screen, BLUE, [350, 145], [450,245], 10)
        pygame.draw.line(screen, BLUE, [450, 145], [350,245], 10)
    else:
        pygame.draw.circle(screen, RED, [400, 195], 50, 10)
    GridGame[1] = player
    return 1 + player * (-1)
   
#2
def GridGame2(screen):
    if player == 1:
        pygame.draw.line(screen, BLUE, [500, 145], [600,245], 10)
        pygame.draw.line(screen, BLUE, [600, 145], [500,245], 10)
    else:
        pygame.draw.circle(screen, RED, [550, 195], 50, 10)
    GridGame[2] = player
    return 1 + player * (-1)
   
#3
def GridGame3(screen):
    if player == 1:
        pygame.draw.line(screen, BLUE, [200, 295], [300,395], 10)
        pygame.draw.line(screen, BLUE, [300, 295], [200,395], 10)
    else:
        pygame.draw.circle(screen, RED, [250, 345], 50, 10)
    GridGame[3] = player
    return 1 + player * (-1)
   
#4
def GridGame4(screen):
    if player == 1:
        pygame.draw.line(screen, BLUE, [350, 295], [450,395], 10)
        pygame.draw.line(screen, BLUE, [450, 295], [350,395], 10)
    else:
        pygame.draw.circle(screen, RED, [400, 345], 50, 10)
    GridGame[4] = player
    return 1 + player * (-1)
   
#5
def GridGame5(screen):
    if player == 1:
        pygame.draw.line(screen, BLUE, [500, 295], [600,395], 10)
        pygame.draw.line(screen, BLUE, [600, 295], [500,395], 10)
    else:
        pygame.draw.circle(screen, RED, [550, 345], 50, 10)
    GridGame[5] = player
    return 1 + player * (-1)
   
#6
def GridGame6(screen):
    if player == 1:
        pygame.draw.line(screen, BLUE, [200, 445], [300,545], 10)
        pygame.draw.line(screen, BLUE, [300, 445], [200,545], 10)
    else:
        pygame.draw.circle(screen, RED, [250, 495], 50, 10)
    GridGame[6] = player
    return 1 + player * (-1)
   
#7
def GridGame7(screen):
    if player == 1:
        pygame.draw.line(screen, BLUE, [350, 445], [450,545], 10)
        pygame.draw.line(screen, BLUE, [450, 445], [350,545], 10)
    else:
        pygame.draw.circle(screen, RED, [400, 495], 50, 10)
    GridGame[7] = player
    return 1 + player * (-1)
   
#8
def GridGame8(screen):
    if player == 1:
        pygame.draw.line(screen, BLUE, [500, 445], [600,545], 10)
        pygame.draw.line(screen, BLUE, [600, 445], [500,545], 10)
    else:
        pygame.draw.circle(screen, RED, [550, 495], 50, 10)
    GridGame[8] = player
    return 1 + player * (-1)

def Winner(player, x_start, y_start, x_end, y_end, screen):
    if player == 1:
        pygame.draw.line(screen, BLUE, [x_start, y_start], [x_end,y_end], 20)
    else:
        pygame.draw.line(screen, RED, [x_start, y_start], [x_end,y_end], 20)

def WinnerTest(player, line, screen):
    if (GridGame[0] == player and GridGame[1] == player and GridGame[2] == player): # across the top
        if line == 1 :
            Winner(player,175,195,625,195, screen)
            GameResult(player, screen)
        return True
    if (GridGame[3] == player and GridGame[4] == player and GridGame[5] == player): # across the middle
        if line == 1 :
            Winner(player,175,345,625,345, screen)
            GameResult(player, screen)
        return True    
    if (GridGame[6] == player and GridGame[7] == player and GridGame[8] == player): # across the bottom
        if line == 1 :
            Winner(player,175,495,625,495, screen)
            GameResult(player, screen)
        return True
    if (GridGame[0] == player and GridGame[3] == player and GridGame[6] == player): # down the left side
        if line == 1 :
            Winner(player,250,120,250,570, screen)
            GameResult(player, screen)
        return True
    if (GridGame[1] == player and GridGame[4] == player and GridGame[7] == player): # down the middle
        if line == 1 :
            Winner(player,400,120,400,570, screen)
            GameResult(player, screen)
        return True
    if (GridGame[2] == player and GridGame[5] == player and GridGame[8] == player): # down the right side
        if line == 1 :
            Winner(player,550,120,550,570, screen)
            GameResult(player, screen)
        return True
    if (GridGame[0] == player and GridGame[4] == player and GridGame[8] == player): # diagonal
        if line == 1 :
            Winner(player,175,120,625,570, screen)
            GameResult(player, screen)
        return True
    if (GridGame[2] == player and GridGame[4] == player and GridGame[6] == player): # diagonal
        if line == 1 :
            Winner(player,625,120,175,570, screen)
            GameResult(player, screen)
        return True    
    return False

def ClearGridGame():
    #drawScreen()
    # Reset the Game Grid
    for i in range(9):
        GridGame[i] = ' '

def isGridGameFull():
    # Return True if every space on the board has been taken. Otherwise return False.
    for i in range(9):
        if GridGame[i] == ' ':
            return False
    return True

def GameResult(player, screen):
    global n_player
    global n_computer
    if player==0:
        text = "The Computer Won!!!"
        n_computer += 1
    if player==1:
        text = "The Player Won!!!"
        n_player += 1
    if player==2:
        text = "The Game is Tie!!!"
    surf = utils.char2surf(text,72,BLACK)
    screen.blit(surf,(100,300))
    pygame.display.flip()
    pygame.time.wait(2000)
    pygame.event.clear()

def Score(screen):
    surf = utils.char2surf("Player: " + str(n_player),18,BLUE)
    screen.blit(surf,(20,570))
    surf = utils.char2surf("Computer: " + str(n_computer),18,RED)
    screen.blit(surf,(680,570))
    

def chooseRandomMoveFromList(listmoves):
    # Choose a move on the list is is possible
    moves = []
    for i in listmoves:
        if GridGame[i] == ' ':
            moves.append(i)

    if len(moves) != 0:
        return random.choice(moves)
    else:
        return None

def ComputerMove(player, screen):

    # Check if the computer can win in the next move
    for i in range(9):
        if GridGame[i] == ' ':
            GridGame[i] = player
            if WinnerTest(player, 0, screen):
                GridGame[i] = ' '
                return i
            GridGame[i] = ' '

    # Check if the player could win on his next move, and block them.
    for i in range(9):
        if GridGame[i] == ' ':
            GridGame[i] = 1 + player * (-1)
            if WinnerTest(1 + player * (-1), 0, screen):
                GridGame[i] = ' '
                return i
            GridGame[i] = ' '

    # Try to move to the corners, if they are free.
    move = chooseRandomMoveFromList([0, 2, 6, 8])
    if move != None:
        return move

    # Try to take the center, if it is free.
    if GridGame[4] == ' ':
        return 4

    # Move on one of the sides.
    return chooseRandomMoveFromList([1, 3, 5, 7])

# Reset the Game
GridGame = [' '] * 9

# who Plays First
player = whoPlaysFirst()

n_player = 0
n_computer = 0



class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("schoolsplay.TicTacToe.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','TictactoeData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'TicTacToe.rc'))
        self.logger.debug("rchash: " + str(self.rchash))
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
    
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.refresh()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Tictactoe")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "TicTacToe"
    
    def get_help(self):
        """Mandatory methods"""
        text = ["Objective:",
        " ",
        "Try to complete, first, a row of 3 matching symbols,",
        "horizontally, vertically or diagonally.",
        " ",
        "Tip:",
        " ",
        "If your opponent has two symbols on the same line,",
        "you must put your symbol on that line to block him.",
        " "]

        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")
    
    def start(self):
        """Mandatory method."""
        pass
        
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        #return _("This activity has %s levels") % 6
        return _("This activity has no levels")
    
    def pre_level(self,level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        self.logger.debug("pre_level called with: %s" % level)
        pass
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("nextlevel called with: %s" % level)
        # Your top blit position, this depends on the menubar position 
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        # Remove these four lines
        #text = "Template activity for " + self.get_helptitle()
        #surf = utils.char2surf(text,24,RED)
        #self.screen.blit(surf,(20,200))

        drawGrid(self.screen)

        return True
    

    
    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        pass
        
    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        pass
        return True
        
        
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass

    def loop(self,events):
        global player
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""

        if player == 0:
            move = ComputerMove(player, self.screen)
            if move == 0:
                player = GridGame0(self.screen)
            elif move == 1:
                player = GridGame1(self.screen)
            elif move == 2:
                player = GridGame2(self.screen)
            elif move == 3:
                player = GridGame3(self.screen)
            elif move == 4:
                player = GridGame4(self.screen)
            elif move == 5:
                player = GridGame5(self.screen)
            elif move == 6:
                player = GridGame6(self.screen)
            elif move == 7:
                player = GridGame7(self.screen)
            else:
                player = GridGame8(self.screen)

        for event in events:
            if event.type is MOUSEBUTTONDOWN:
                pos_x = event.pos[0]
                pos_y = event.pos[1]
                if pos_x > 175 and pos_x <325:
                    if pos_y > 120 and pos_y < 270:
                        #0
                        if GridGame[0] == ' ':
                            player = GridGame0(self.screen)
                    elif pos_y > 270 and pos_y < 420:
                        #3
                        if GridGame[3] == ' ':
                            player = GridGame3(self.screen)
                    elif pos_y > 420 and pos_y < 570:
                        #6
                        if GridGame[6] == ' ':
                            player = GridGame6(self.screen)
                elif pos_x > 325 and pos_x <475:
                    if pos_y > 120 and pos_y < 270:
                        #1
                        if GridGame[1] == ' ':
                            player = GridGame1(self.screen)
                    elif pos_y > 270 and pos_y < 420:
                        #4
                        if GridGame[4] == ' ':
                            player = GridGame4(self.screen)
                    elif pos_y > 420 and pos_y < 570:
                        #7
                        if GridGame[7] == ' ':
                            player = GridGame7(self.screen)
                elif pos_x > 475 and pos_x <625:
                    if pos_y > 120 and pos_y < 270:
                        #2
                        if GridGame[2] == ' ':
                            player = GridGame2(self.screen)
                    elif pos_y > 270 and pos_y < 420:
                        #5
                        if GridGame[5] == ' ':
                            player = GridGame5(self.screen)
                    elif pos_y > 420 and pos_y < 570:
                        #8
                        if GridGame[8] == ' ':
                            player = GridGame8(self.screen)

        if WinnerTest(1 + player * (-1), 1, self.screen):
            self.clear_screen()
            ClearGridGame()
            drawGrid(self.screen)

        if isGridGameFull():
            GameResult(2, self.screen)
            self.clear_screen()
            ClearGridGame()
            drawGrid(self.screen)

        pygame.display.flip()

        return

    
        



"""
Borg and more than one module, Jan Wender, 2001/10/11
If you want to use the Borg pattern from more than one module,
it works as follows: Extract the Borg Pattern into a module and a
class of its own: borg.py:

Let your class in a separate module inherit from Borg and call Borg's
initializer in your __init__:

import BorgSingleton
class Test(BorgSingleton.Borg):
  def __init__(self):
     BorgSingleton.Borg.__init__(self)

Although sounding like a swedish chef, it works now.
The state is shared across all modules using the Test class. 
"""

class Borg:
    __shared_state = {}
    def __init__(self):
        self.__dict__ = self.__shared_state
    # and whatever else you want in your class -- that's all!
    



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SQLTables.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Put all the tables that are used in the dbase here

# ACTIVITY DEVELOPERS, BE SURE YOU UNDERSTAND WHAT YOU MUST ADD HERE TO GET YOUR
# ACTIVITY UP AND RUNNING.
# PLEASE CONTACT THE AUTHOR OF THIS FILE OR THE SCHOOLSPLAY MAILINGLIST FOR
# ADDITIONAL INFO OR HELP.

# PLEASE DON'T CHANGE ANYTHING IN THIS MODULE WITHOUT DISCUSSING IT ON THE
# SCHOOLSPLAY MAILINGLIST.
# THE CreateTemplate SCRIPT and Datamanager DEPENDS ON CERTAIN SYNTAX TO BE 
# PRESENT IN HERE.

# Make sure to add a ORM class to SPORMs.py if you add new sql tables.
import datetime
from sqlalchemy import *
import sqlalchemy.orm as sqlorm
    
import SPORMs as ORMS

def create_contentdb_orms(metadata):
    tables = {}
    for name in [k for k in ORMS.__dict__.keys() if k.startswith('game_')]:
        t = Table(name, metadata, autoload=True)
        orm = getattr(ORMS, name)
        orm._name = name
        sqlorm.mapper(orm, t)
        tables[name] = orm
    return tables

class SqlTables:
    """Class that holds the dbase table definitions and creates the tables for
    the content dbase and user dbase."""
    def _create_userdb_tables(self, metadata):
        """Called by the datamanager to setup the dbase tables.
        We also map the ORM classes to the dbase tables.
        Returns a dict with the keys set to the dbase table names and
        the values set to the ORM classes."""
        tables = {}
        sqlorm.clear_mappers()
        for table in self.tableslist:
            self._appendcolumns(table)
            if hasattr(ORMS, table.name):
                orm = getattr(ORMS, table.name)
                orm._name = table.name
                sqlorm.mapper(orm, table)
                tables[table.name] = orm
        
        metadata.create_all()
        return tables
    
    def _appendcolumns(self, t):
        # These are the skipped Tables for default columns
        if t.name in ['users', 'options', 'activity_options', 'menu', \
                    'served_content', 'group_names', 'change_pass', 'zorgenquete', \
                    'spconf',  'users_faces', 'dt_sequence', 'stats', 'stats_session']:
            return 
        # Columns added to all act tables
        t.append_column(Column('table_id', Integer, primary_key=True))
        t.append_column(Column('user_id', Integer, nullable=False))
        t.append_column(Column('timespend', Unicode(6)))
        t.append_column(Column('start_time', Unicode(17)))
        t.append_column(Column('end_time', Unicode(17)))
        t.append_column(Column('level', Integer))
        t.append_column(Column('score', Float))
                
    def __init__(self, metadata):
        # table to hold user data, mandatory for any sp dbase.
        # All other tables are regarded as activity tables.
        # Used by the core, don't change it
        self.users = Table('users', metadata,\
            Column('user_id', Integer, primary_key=True), \
            Column('login_name', Unicode(20), unique=True, nullable=False), \
            Column('title', Unicode(20)), \
            Column('first_name', Unicode(20)),\
            Column('last_name', Unicode(40)), \
            Column('birthdate', DateTime), \
            Column('group', Integer), \
            Column('profile', Unicode(200)), \
            Column('passwrd', Unicode(10)), \
            Column('activities', Unicode(250)), \
            Column('audio', Integer, default=50),\
            Column('dt_target', Unicode(250), default='default'),\
            Column('levelup_dlg', Unicode(4), default='true')\
            )
    
        # The graph image needs two values:
        # average (greek: mu-value) and the standard deviation (greek: sigma-value)
        # mu is a integer between 1-10 and sigma is an integer between 0-100 which
        # stand for the percentage deviation. (Probably always 50%)
        self.activity_options = Table('activity_options', metadata, \
            Column('table_id', Integer, primary_key=True), \
            Column('activity', Unicode(20)), \
            Column('mu', Float, default=6.5), \
            Column('sigma', Float, default=1.63)\
            )
        
        # served content table used by the acts to store content they serve so that
        # we can make sure we don't serve the same content twice.
        # This is meanly intended for the quizzes as the rest of the acts don't
        # have much content, yet ?
        self.served_content = Table('served_content', metadata, \
                Column('table_id', Integer, primary_key=True), \
                Column('user_id', Integer,nullable=False), \
                Column('CID', Integer, nullable=False),\
                Column('game_theme_id', Integer, nullable=False),\
                Column('module', Unicode(25)), \
                Column('start_time', DATETIME(timezone=True)), \
                Column('count_served', Integer, default=0), \
                Column('wrong', Integer, default=0)
                )

        # Table to convert group name to an id
        self.group_names = Table('group_names', metadata, \
                                 Column('group_id', Integer, primary_key=True), \
                                 Column('group_name', Unicode(50), unique=True, nullable=False))
        
        self.spconf = Table('spconf', metadata, \
                            Column('table_id', Integer, primary_key=True), \
                            Column('activity_name', Unicode(50)), \
                            Column('key', Unicode(20)), \
                            Column('value', Unicode(200)), \
                            Column('theme', Unicode(20), default='default'), \
                            Column('comment', Unicode(200)))        

        self.faces = Table('users_faces',  metadata, \
                           Column('face_id',  Integer, primary_key=True),  \
                           Column('user_id',  Integer),  \
                           Column('shape_diff', Float), \
                           Column('temp_diff', Float), \
                           Column('pic_name',  Unicode(10)),  \
                           Column('datetime', DATETIME))

        self.stats = Table('stats', metadata,\
                           Column('ID', Integer, primary_key=True),\
                           Column('datetime', DATETIME),\
                           Column('activity_name', Unicode(50)),\
                           Column('message', Unicode(255)),\
                           Column('game_start_called', Boolean, default=False),\
                           Column('game_nextlevel_called', Boolean, default=False),\
                           Column('game_end_called', Boolean, default=False),\
                           Column('is_cop', Boolean, default=False),\
                           Column('user_id', Integer),\
                           Column('final_score', Integer),\
                           Column('session', Integer))
        self.stats_session = Table('stats_session', metadata,\
                            Column('ID', Integer, primary_key=True),\
                            Column('datetime', DATETIME))


        ################ Activity tables ########################## 
        # Every activity *must* have a sql table set.
        # The name of the table must be the same as the activity modules name
        # The following cols are mandatory for any activity table.
        # 'table_id' to hold the id
        # 'user_id' to hold the user id (set by DM)
        # 'timespend' to hold the time it took to complete the level. (set by core)
        # 'start_time' to hold the time the level is started (set by core)
        # 'end_time' to hold the time the level is finished (set by core)
        # 'level' to hold the level number (set by core)
        # 'score' to hold the score value.
        # The mandatory columns are provided by the BaseTable class, 
        # the reporter tool expect them to be present so please extend the BaseTable
        # instead of creating your own.
        # The rest of the cols are set/used by the activity.
        
        # LEAVE THESE COMMENTS IN HERE, THE CreateTemplate SCRIPT NEEDS THEM.
        # @splitpoint@
        self.flashcards = Table('flashcards',metadata)
        self.BlockBreaker = Table('BlockBreaker',metadata)
        self.TicTacToe = Table('TicTacToe',metadata)
        
        self.fourrow = Table('fourrow',metadata,\
                Column('stonesplayed', Integer),\
                    )
        self.numbers_sp = Table('numbers_sp',metadata)
        
        
        self.test_act = Table('test_act',metadata)
        
        self.fishtank = Table('fishtank',metadata,\
                Column('fish', Integer),\
                Column('clearfish', Integer),\
                    )
        self.ichanger = Table('ichanger',metadata,\
                Column('wrongs', Integer),\
                Column('extra', Integer),\
                    )
        self.puzzle = Table('puzzle',metadata,\
                Column('total_pieces', Integer),\
                Column('wrongs', Integer),\
                    )
        
        self.simon_sp = Table('simon_sp',metadata,\
                Column('sounds', Integer))
        
        self.soundmemory = Table('soundmemory', metadata, \
            Column('sounds', Integer), \
            Column('knownsounds', Integer), \
                )
        
        self.memory_sp = Table('memory_sp',metadata,\
                Column('cards', Integer),\
                Column('knowncards', Integer),\
                    )
        self.quiz_royal = Table('quiz_royal',metadata,\
                Column('total_questions', Integer),\
                Column('total_wrongs', Integer),\
                    )
        self.quiz_history = Table('quiz_history',metadata,\
                Column('total_questions', Integer),\
                Column('total_wrongs', Integer),\
                    )
        self.quiz_picture = Table('quiz_picture',metadata,\
                Column('total_questions', Integer),\
                Column('total_wrongs', Integer),\
                    )
        self.quiz_math = Table('quiz_math',metadata,\
                Column('total_questions', Integer),\
                Column('total_wrongs', Integer),\
                    )
        self.quiz_text = Table('quiz_text',metadata,\
                Column('total_questions', Integer),\
                Column('total_wrongs', Integer),\
                    )
        self.quiz = Table('quiz',metadata,\
                Column('total_questions', Integer),\
                Column('total_wrongs', Integer),\
                    )
        self.quiz_general = Table('quiz_general',metadata,\
                Column('total_questions', Integer),\
                Column('total_wrongs', Integer),\
                    )
        self.quiz_melody = Table('quiz_melody',metadata,\
                Column('total_questions', Integer),\
                Column('total_wrongs', Integer),\
                    )
        self.quiz_sayings = Table('quiz_sayings',metadata,\
                Column('total_questions', Integer),\
                Column('total_wrongs', Integer),\
                    )
        self.findit_sp = Table('findit_sp',metadata,\
                Column('total_diffs', Integer),\
                Column('wrongs', Integer),\
                    )
                    
        self.electro_sp = Table('electro_sp', metadata, \
            Column('knowncards', Integer), \
            Column('cards', Integer))
            
        self.video = Table('video', metadata, \
            Column('movie', Unicode(20)), \
            Column('answer', Integer))
        
        self.dt_sequence = Table('dt_sequence',metadata,\
                 Column('fortune', Integer),\
                 Column('act_name', Unicode(255)),\
                 Column('group', Unicode(255)),\
                 Column('level', Integer),\
                 Column('cycles', Integer),\
                 Column('target', Unicode(255)),\
                 Column('table_id', Integer, primary_key=True),\
                 Column('order', Integer))
        
        #TODO: This table is not finished, as soon as the specs are known we must update this
        self.dltr = Table('dltr', metadata, \
            Column('activity', Unicode(20)), \
            Column('done', Integer), \
            Column('cycles', Integer), \
            Column('epoch', Numeric))
                        
        ############ Setup an activity list #################################
        # Now we put all the tables we have into this list.
        # The Datamanager uses this list to setup the dbase tables.
        self.tableslist = [self.users, self.activity_options,self.served_content, \
                           self.spconf, self.stats,self.stats_session,\
                           self.group_names, self.findit_sp, self.electro_sp,\
                        self.quiz_text, self.quiz_melody, self.quiz_math,\
                        self.quiz_picture, self.quiz_history, self.quiz_royal,\
                        self.quiz, self.quiz_general, self.quiz_sayings, \
                        self.memory_sp, self.simon_sp, self.puzzle,\
                        self.soundmemory, self.ichanger, self.fishtank,\
                        self.dltr, \
                        self.test_act,\
                        self.numbers_sp,\
                        self.fourrow, self.dt_sequence
                        ,\
                        self.TicTacToe,\
                        self.BlockBreaker,\
                        self.flashcards]









if __name__ == '__main__':
    # test code
    from SPConstants import DBASEPATH
    import sqlalchemy as sqla
    engine = sqla.create_engine('sqlite:///%s' % DBASEPATH)
    metadata = sqla.MetaData(engine)
    #metadata.engine.echo = True
    sqltb = SqlTables(metadata)
    tables = sqltb._create_tables(metadata)
    print tables
    



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           photoalbum.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.photoalbum.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.photoalbum")

# standard modules you probably need
import os, glob, random
import shelve, textwrap
import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
import SPWidgets
from Timer import Timer

try:
    from xml.etree.ElementTree import ElementTree
except ImportError:
    # try the python2.4 way 
    from elementtree.ElementTree import ElementTree

class Photo(SPSpriteUtils.SPSprite):
    def __init__(self, p):
        self.image = utils.load_image(p)
        SPSpriteUtils.SPSprite.__init__(self, self.image)
        pos = ((800 - self.rect.w) / 2, (600 - self.rect.h) / 2)
        self.moveto(pos)
        
class TextDisplay(SPSpriteUtils.SPSprite):
    def __init__(self, text, surf):
        self.image = surf
        tsurf = utils.char2surf(text, fsize=14, fcol=BLACK)
        self.image.blit(tsurf, (10, 3))
        SPSpriteUtils.SPSprite.__init__(self, self.image)
        self.moveto((0, 500))
        self.set_use_current_background(True)
        

# This activity differs from standard CP acts in the way that it runs fullscreen,
# disables the menubuttons, uses old fashion screen blitting for it's buttons
# and uses a state checking eventloop pattern.

class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.photoalbum.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        self.blacksurf = pygame.Surface((800, 600)).convert(8)
        self.blacksurf.fill(BLACK)
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','PhotoalbumData')
        self.thumbsdir = os.path.join(self.my_datadir, 'thumbs')
        if not os.path.exists(self.thumbsdir):
            os.mkdir(self.thumbsdir)
        self.shelvepath = os.path.join(self.thumbsdir, 'thumbs.db')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        self.importDir = os.path.join(HOMEDIR, 'braintrainer', 'PicImport', self.lang)
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'photoalbum.rc'))
        self.rchash['theme'] = self.theme
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        
        self.timer = None
        self.timerpause = int(self.rchash[self.theme]['pause'])
        # We use old fashion surfs to blit on the screen iso buttons as transbuttons
        # with dynamic background would store a complete background surface each in memory!!
        quit_surf = utils.load_image(os.path.join(self.my_datadir, 'quit.icon.png'))
        quitro_surf = utils.load_image(os.path.join(self.my_datadir, 'quit_ro.icon.png'))
        main_exit_pos = (800 - quit_surf.get_size()[0], 0)
        self.main_exit_but = SPWidgets.TransImgButton(quit_surf, quitro_surf, pos=main_exit_pos)
        self.main_exit_but.connect_callback(self.main_exit_but_cbf, MOUSEBUTTONDOWN)
        self.main_exit_but.set_use_current_background(True)
        #self.main_exit_rect = pygame.Rect(self.main_exit_pos + self.main_exit_surf.get_size())
        
        exit_surf = utils.load_image(os.path.join(self.my_datadir, 'category.icon.png'))
        exitro_surf = utils.load_image(os.path.join(self.my_datadir, 'category_ro.icon.png'))
        exit_pos = (0, 0)
        self.exit_but = SPWidgets.TransImgButton(exit_surf, exitro_surf, pos=exit_pos)
        self.exit_but.connect_callback(self.exit_but_cbf, MOUSEBUTTONDOWN)
        self.exit_but.set_use_current_background(True)
        
        play_surf = utils.load_image(os.path.join(self.my_datadir, 'play.icon.png'))
        playro_surf = utils.load_image(os.path.join(self.my_datadir, 'play_ro.icon.png'))
        
        play_pos = (0, 490 - play_surf.get_size()[1])
        self.play_but = SPWidgets.TransImgButton(play_surf, playro_surf, pos=play_pos)
        self.play_but.connect_callback(self.play_but_cbf, MOUSEBUTTONDOWN)
        #self.play_but.set_use_current_background(True)

        text_surf = utils.load_image(os.path.join(self.my_datadir, 'text.icon.png'))
        textro_surf = utils.load_image(os.path.join(self.my_datadir, 'text_ro.icon.png'))
        text_pos = (800 - text_surf.get_size()[0], 490 - text_surf.get_size()[1])
        self.text_but = SPWidgets.TransImgButton(text_surf, textro_surf, pos=text_pos)
        self.text_but.connect_callback(self.text_but_cbf, MOUSEBUTTONDOWN)
        self.text_but.set_use_current_background(True)

        self.buttons_list = [self.play_but, self.main_exit_but, self.exit_but]
        
        self.text_label = None
        self.text_back = utils.load_image(os.path.join(self.my_datadir, 'text_backgr.png'))
        self.textstring = ''
        
        self.prev_mouse_pos = None
        self.SPG.tellcore_set_dice_minimal_level(6)
        
        self.clear_screen()
    
    def __del__(self):
        self.SPG.tellcore_enable_menubuttons()
    
    def clear_screen(self):
        self.screen.blit(self.blacksurf, (0, 0))
        self.backgr.blit(self.blacksurf, (0, 0))
        pygame.display.update()

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.refresh()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Photoalbum")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "photoalbum"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
        " ",
        _("Number of levels : 1"),
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")
        
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 1

    def start(self):
        """Mandatory method."""
        self.stopme = False
        self.thumbx, self.thumby = eval(self.rchash[self.theme]['thumbsize'], {}, {})
        # check to see if the thumbs dir is up to date
        tlist = os.listdir(self.thumbsdir)
        # first we look for language specific albums.
        # TODO: look for import foto's in schoolsplay.rc
        self.logger.debug("looking for %s_Album_* " % self.lang)
        albums = glob.glob(os.path.join(self.my_datadir, '%s_Album_*' % self.lang))
        if not albums:
            self.logger.debug("looking for *Album_* ")
            albums = glob.glob(os.path.join(self.my_datadir, '*Album_*'))
        if not albums:
            self.logger.debug("looking for *_Album_* ")
            glob.glob(os.path.join(self.my_datadir, '*_Album_*'))
        if not albums:
            self.logger.debug("looking for *Album*")
            glob.glob(os.path.join(self.my_datadir, '*Album*'))
        # Check if we have custom imported pictures
        self.logger.debug("looking for imported pictures in %s" % self.importDir)
        extra_albums = glob.glob(os.path.join(self.importDir, '%s_Album*' % self.lang))
        if extra_albums:
            albums += extra_albums
        if not albums:
            self.SPG.tellcore_info_dialog(_("No photo albums found !\nYou should place your pictures inside directories called 'Album_1', 'Album_2' etc\n Place these directories inside %s" % self.my_datadir))
            self.stopme = True
        # check that we don't use empty albums.
        self.logger.debug("found albums: %s" % albums)
        sh = shelve.open(self.shelvepath)
        sh_keys = sh.keys()
        for d in albums:
            tname = os.path.split(d)[1].encode('utf-8')
            flist = os.listdir(d)
            flist.sort()
            if not flist or os.path.splitext(flist[0])[1] not in IMAGE_EXT:
                continue
            if tname + '.jpeg' not in tlist or not sh.has_key(tname):
                self.logger.debug("no thumb found for album %s, creating one" % tname)
                img = utils.load_image(os.path.join(d, flist[0]))
                tsurf = utils.aspect_scale(img, (self.thumbx, self.thumby))
                p = os.path.join(self.thumbsdir, tname + '.jpeg')
                pygame.image.save(tsurf, p)
                self.logger.debug("%s thumb saved to %s" % (tname, p))
                sh[tname] = d
            elif sh.has_key(tname):
                sh_keys.remove(tname)
        if sh_keys:
            # we have thumbs but no albums, this is caused by the removal of albums.
            for key in sh_keys:
                if sh.has_key(key):
                    del sh[key]
                p = os.path.join(self.SPG.get_base_dir(), self.thumbsdir, key + '.jpeg')
                if os.path.exists(p):
                    self.logger.debug("removing thumb %s" % p)
                    os.remove(p)
        sh.close() 
    
    def pre_level(self, level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.SPG.tellcore_disable_menubuttons()
        self.clear_screen()
        
        if level > 1:
            return
        return True
    
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        self.photoindex = -1
        self.currentphoto = None
        self.button_state = 'show'
        self.hide_buttons_timer = None
        self.button_action = False
        self.show_thumbs()
        
    def show_thumbs(self):
        self.stop_mytimer()
        self.clear_screen()
        self.actives.empty()
        self.main_exit_but.display_sprite()
        self.actives.add(self.main_exit_but)
        self.state = 'thumbs'
        self.thumbslist =[]
        
        x = 20
        flist = glob.glob(os.path.join(self.thumbsdir, '%s_*.jpeg' % self.lang))
        flist.sort()
        # Your top blit position, this depends on the menubar position 
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        sh = shelve.open(self.shelvepath)
        for f in flist:
            key = os.path.splitext(os.path.split(f)[1])[0].encode('utf-8')
            img = utils.load_image(f)
            if sh.has_key(key):
                b = SPWidgets.SimpleButton(img, (x, y), data=sh[key])
            else:
                self.logger.error("missing shelve key: %s" % key)
                continue
            b.display_sprite()
            self.thumbslist.append(b)
            x += img.get_size()[0] + 30
            if x > 790 - self.thumbx:
                y += self.thumby + 30
                x = 20
        self.actives.add(self.thumbslist)
        sh.close()
    
    def slideshow(self):
        """Start slideshow"""
        self.logger.debug("Start slideshow")
        self.text_but.erase_sprite()
        self.actives.remove(self.text_but)
        self.timer = Timer(self.timerpause,self.next_photo,\
                                lock=self.SPG.get_thread_lock(), startnow=False)
        self.timer.start()
        self.state = 'play'
    
    def stop_slide_show(self):
        self.logger.debug("Stop slideshow")
        self.stop_mytimer()
        self.state = 'manual'
        self.show_all_buttons()
        
    def show(self):
        """Start show photos one at the time"""
        self.clear_screen()
        self.button_state = 'hide'
        self.stop_mytimer()
        self.state = 'manual'
        self.next_photo()
    
    def next_photo(self):
        # Load photo sprites at run time to prevent large amounts of memory usage.
        if self.photoindex >= len(self.filelist) - 1:
            self.photoindex = 0
        else:
            self.photoindex += 1
#        effects = (self.wipeleft2right, self.wiperight2left)
#        apply(random.choice(effects), (self.filelist[self.photoindex], ))
          
#        print "effects finished"
        if self.currentphoto:
            self.currentphoto.erase_sprite()
        if self.text_label:
            self.text_label.erase_sprite()
            self.text_label = None
        
        p = self.filelist[self.photoindex]
        if self.textxml and self.textxml.has_key(os.path.basename(p)):
            hash = self.textxml[os.path.basename(p)]
            title = "%s: %s" % (self.textxml['album_name'], hash['title'])
            text = "\n".join(textwrap.wrap(hash['text'], 90))
            
            self.textstring = "%s\n%s" % (title, text)
        else:
            self.textstring = ''    
            
        self.currentphoto = Photo(p)
        self.currentphoto.display_sprite()
        if self.state == 'text':
            self.state = 'manual'
        if self.state == 'manual':
            self.show_all_buttons()
        return True
    
    def prev_photo(self):
        if self.photoindex == 0:
            self.photoindex = len(self.filelist) - 1
        else:
            self.photoindex -= 1
        if self.currentphoto:
            self.currentphoto.erase_sprite()
        if self.text_label:
            self.text_label.erase_sprite()
            self.text_label = None
        p = self.filelist[self.photoindex]
        if self.textxml and self.textxml.has_key(os.path.basename(p)):
            hash = self.textxml[os.path.basename(p)]
            title = "%s: %s" % (self.textxml['album_name'], hash['title'])
            text = "\n".join(textwrap.wrap(hash['text'], 90))
            
            self.textstring = "%s\n%s" % (title, text)
        else:
            self.textstring = ''    
        self.currentphoto = Photo(p)
        self.currentphoto.display_sprite()
        if self.state == 'manual':
            self.show_all_buttons()

    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
    
    def stop_mytimer(self):
        try:
            self.timer.stop()
        except:
            pass
        try:
            self.hide_buttons_timer.stop()
        except:
            pass
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        self.SPG.tellcore_enable_menubuttons()

    def main_exit_but_cbf(self, *args):
        self.button_action = True
        self.stop_mytimer()
        self.SPG.tellcore_enable_menubuttons()
        self.SPG.tellcore_game_end()

    def exit_but_cbf(self, *args):
        self.button_action = True
        self.stop_mytimer()
        if self.state != 'thumbs':
            self.hide_all_buttons()
            self.show_thumbs()

    def play_but_cbf(self, *args):
        self.button_action = True
        if self.state == 'manual':
            self.state = 'play'
            self.prev_mouse_pos = 0
            self.hide_all_buttons()
            self.slideshow()
    
    def text_but_cbf(self, *args):
        if self.state == 'play':
            return
        if self.text_label:
            self.text_label.erase_sprite()
            self.text_label = None
            self.state = 'manual'
            self.show_all_buttons()
        else:
            #self.play_but.erase_sprite()
            self.text_label = TextDisplay(self.textstring, self.text_back.copy())
            self.text_label.display_sprite()
            self.state = 'text'

    def hide_all_buttons(self):
        if self.button_state == 'hide':
            return
        self.button_state = 'hide'
        for b in self.buttons_list:
#            if self.state == 'text' and b is self.play_but:
#                continue
            b.erase_sprite()
        self.text_but.erase_sprite()
        self.actives.remove(self.buttons_list)
        self.actives.remove(self.text_but)
    
    def show_all_buttons(self):
        if self.button_state == 'show':
            return
        if self.hide_buttons_timer:
            self.hide_buttons_timer.stop()
        self.button_state = 'show'
        for b in self.buttons_list:
            b.display_sprite()
        self.actives.add(self.buttons_list)
#        if self.state == 'text':
#            self.actives.remove(self.play_but)
#            self.play_but.erase_sprite()
        if self.state == 'play':
            self.actives.remove(self.text_but)
            self.text_but.erase_sprite()
        else:
            if self.textstring:
                self.text_but.display_sprite()
                self.actives.add(self.text_but)
        #self.hide_buttons_timer = Timer(2.0, self.hide_all_buttons,\
         #                       lock=self.SPG.get_thread_lock(), startnow=False)
        #self.hide_buttons_timer.start()
    
    def parse_xml(self, p):
        """Parses the whole xml tree into a hash with lists. Each list contains hashes
        with the elelements from a 'activity' element.
        """  
        self.logger.debug("Starting to parse: %s" % p)
        if not os.path.exists(p):
            return {}
        tree = ElementTree()
        try:
            tree.parse(p)
        except Exception, info:
            self.logger.error("%s" % info)
            raise utils.MyError, info
        # here we start the parsing
        albumNode = tree.getroot()
        album_name = albumNode.get('name')
        album_numpics = albumNode.get('nofPicture')
        
        photoNodes = tree.findall('photo')
        self.logger.debug("found %s photo nodes in total" % len(photoNodes))
        albumhash = {'album_name': album_name, 'album_numpics':album_numpics}
        for node in photoNodes:
            hash = {}
            try:
                hash['name'] = node.get('name')
                hash['title'] = node.find('title').text
                hash['text'] = node.find('text').text
            except AttributeError, info:
                self.logger.error("The %s is badly formed, missing element(s):%s,%s" % (xml, info, e))
                albumhash = {}
            else:
                albumhash[hash['name']] = hash
        #self.logger.debug("xml hash:%s" % albumhash)
        return albumhash

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        if self.stopme:
            self.SPG.tellcore_enable_menubuttons()
            self.SPG.tellcore_game_end()
        for event in events:
            if event.type in (MOUSEBUTTONDOWN, MOUSEBUTTONUP,  MOUSEMOTION):
                result = self.actives.update(event)
                if self.button_action:
                    self.button_action = False
                    return
                if event.type == MOUSEBUTTONDOWN:
                    if self.state == 'play':
                        self.stop_slide_show()
                        self.prev_mouse_pos = 0
                        return
                    elif self.state == 'thumbs':
                        if result:
                            self.logger.debug("loop, got a thumb")
                            self.state = 'show'
                            album = result[0][1][0]
                            xml = os.path.join(album, 'text.xml')
                            self.textxml = self.parse_xml(xml)# could return None
                            self.actives.remove(self.thumbslist)
                            self.filelist = [f for f in utils.find_files(album, IMAGE_EXT_PATTERN)]
                            self.filelist.sort()
                            self.show()
                            return
                    elif self.state != 'thumbs':
                        self.show_all_buttons()
                        self.prev_mouse_pos = pygame.mouse.get_pos()[0]
                # gestures
                if event.type == MOUSEBUTTONUP and self.prev_mouse_pos and not self.state == 'thumbs':
                    x = pygame.mouse.get_pos()[0]
                    if self.prev_mouse_pos - x > 20:
                        self.hide_all_buttons()
                        self.next_photo()
                    elif x - self.prev_mouse_pos > 20:
                        self.hide_all_buttons()
                        self.prev_photo()
        return 
        
#    def wipeleft2right(self, p):
#        image = utils.load_image(p)
#        rect = image.get_rect()
#        pos = ((800 - rect.w) / 2, (600 - rect.h) / 2)
#        self.backgr.fill(BLACK)
#        self.backgr.blit(image, pos)
#        for y in range(0, 600, 10):
#            for x in range(0, 800, 10):
#                pygame.time.wait(2)
#                dirty = self.screen.blit(self.backgr, (x, y), (x, y, x+10, y+10))
#                pygame.display.update(dirty)
#
#    def wiperight2left(self, p):
#        image = utils.load_image(p)
#        rect = image.get_rect()
#        pos = ((800 - rect.w) / 2, (600 - rect.h) / 2)
#        self.backgr.fill(BLACK)
#        self.backgr.blit(image, pos)
#        x_range = range(0, 800, 10)
#        x_range.sort(reverse=True)
#        for y in range(0, 600, 10):
#            for x in x_range:
#                pygame.time.wait(2)
#                dirty = self.screen.blit(self.backgr, (x, y), (x-10, y, x, y+10))
#                pygame.display.update(dirty)
#
#   
